@@ -1,4 +1,4 @@
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import
 
 import argparse
 import os
@@ -16,13 +16,10 @@ contributing = os.path.join(
     reporoot, '.hg', 'store', 'data', 'contrib', 'fuzz', 'mpatch.cc.i'
 )
 
-print(changelog, os.path.exists(changelog))
-print(contributing, os.path.exists(contributing))
-
 with zipfile.ZipFile(args.out[0], "w", zipfile.ZIP_STORED) as zf:
     if os.path.exists(changelog):
-        with open(changelog) as f:
+        with open(changelog, 'rb') as f:
             zf.writestr("00changelog.i", f.read())
     if os.path.exists(contributing):
-        with open(contributing) as f:
+        with open(contributing, 'rb') as f:
             zf.writestr("contributing.i", f.read())
