@@ -13,6 +13,7 @@ prints it to ``stderr`` on exit.
 
 from __future__ import absolute_import
 
+
 def memusage(ui):
     """Report memory usage of the current process."""
     result = {'peak': 0, 'rss': 0}
@@ -24,8 +25,13 @@ def memusage(ui):
             key = parts[0][2:-1].lower()
             if key in result:
                 result[key] = int(parts[1])
-    ui.write_err(", ".join(["%s: %.1f MiB" % (k, v / 1024.0)
-                            for k, v in result.iteritems()]) + "\n")
+    ui.write_err(
+        ", ".join(
+            ["%s: %.1f MiB" % (k, v / 1024.0) for k, v in result.iteritems()]
+        )
+        + "\n"
+    )
+
 
 def extsetup(ui):
     ui.atexit(memusage, ui)
