@@ -11,9 +11,12 @@ from mercurial.i18n import _
 
 from mercurial import (
     error,
+    pycompat,
     util,
 )
 
+from mercurial.utils import stringutil
+
 from . import (
     basestore,
     lfutil,
@@ -23,8 +26,10 @@ from . import (
 urlerr = util.urlerr
 urlreq = util.urlreq
 
+
 class remotestore(basestore.basestore):
     '''a largefile store accessed over a network'''
+
     def __init__(self, ui, repo, url):
         super(remotestore, self).__init__(ui, repo, url)
         self._lstore = None
@@ -34,25 +39,32 @@ class remotestore(basestore.basestore):
     def put(self, source, hash):
         if self.sendfile(source, hash):
             raise error.Abort(
-                _('remotestore: could not put %s to remote store %s')
-                % (source, util.hidepassword(self.url)))
+                _(b'remotestore: could not put %s to remote store %s')
+                % (source, util.hidepassword(self.url))
+            )
         self.ui.debug(
-            _('remotestore: put %s to remote store %s\n')
-            % (source, util.hidepassword(self.url)))
+            _(b'remotestore: put %s to remote store %s\n')
+            % (source, util.hidepassword(self.url))
+        )
 
     def exists(self, hashes):
-        return dict((h, s == 0) for (h, s) in # dict-from-generator
-                    self._stat(hashes).iteritems())
+        return dict(
+            (h, s == 0)
+            for (h, s) in pycompat.iteritems(
+                self._stat(hashes)
+            )  # dict-from-generator
+        )
 
     def sendfile(self, filename, hash):
-        self.ui.debug('remotestore: sendfile(%s, %s)\n' % (filename, hash))
+        self.ui.debug(b'remotestore: sendfile(%s, %s)\n' % (filename, hash))
         try:
             with lfutil.httpsendfile(self.ui, filename) as fd:
                 return self._put(hash, fd)
         except IOError as e:
             raise error.Abort(
-                _('remotestore: could not open file %s: %s')
-                % (filename, str(e)))
+                _(b'remotestore: could not open file %s: %s')
+                % (filename, stringutil.forcebytestr(e))
+            )
 
     def _getfile(self, tmpfile, filename, hash):
         try:
@@ -60,15 +72,20 @@ class remotestore(basestore.basestore):
         except urlerr.httperror as e:
             # 401s get converted to error.Aborts; everything else is fine being
             # turned into a StoreError
-            raise basestore.StoreError(filename, hash, self.url, str(e))
+            raise basestore.StoreError(
+                filename, hash, self.url, stringutil.forcebytestr(e)
+            )
         except urlerr.urlerror as e:
             # This usually indicates a connection problem, so don't
             # keep trying with the other files... they will probably
             # all fail too.
-            raise error.Abort('%s: %s' %
-                             (util.hidepassword(self.url), e.reason))
+            raise error.Abort(
+                b'%s: %s' % (util.hidepassword(self.url), e.reason)
+            )
         except IOError as e:
-            raise basestore.StoreError(filename, hash, self.url, str(e))
+            raise basestore.StoreError(
+                filename, hash, self.url, stringutil.forcebytestr(e)
+            )
 
         return lfutil.copyandhash(chunks, tmpfile)
 
@@ -79,17 +96,24 @@ class remotestore(basestore.basestore):
 
     def _verifyfiles(self, contents, filestocheck):
         failed = False
-        expectedhashes = [expectedhash
-                          for cset, filename, expectedhash in filestocheck]
+        expectedhashes = [
+            expectedhash for cset, filename, expectedhash in filestocheck
+        ]
         localhashes = self._hashesavailablelocally(expectedhashes)
-        stats = self._stat([expectedhash for expectedhash in expectedhashes
-                            if expectedhash not in localhashes])
+        stats = self._stat(
+            [
+                expectedhash
+                for expectedhash in expectedhashes
+                if expectedhash not in localhashes
+            ]
+        )
 
         for cset, filename, expectedhash in filestocheck:
             if expectedhash in localhashes:
                 filetocheck = (cset, filename, expectedhash)
-                verifyresult = self._lstore._verifyfiles(contents,
-                                                         [filetocheck])
+                verifyresult = self._lstore._verifyfiles(
+                    contents, [filetocheck]
+                )
                 if verifyresult:
                     failed = True
             else:
@@ -97,30 +121,33 @@ class remotestore(basestore.basestore):
                 if stat:
                     if stat == 1:
                         self.ui.warn(
-                            _('changeset %s: %s: contents differ\n')
-                            % (cset, filename))
+                            _(b'changeset %s: %s: contents differ\n')
+                            % (cset, filename)
+                        )
                         failed = True
                     elif stat == 2:
                         self.ui.warn(
-                            _('changeset %s: %s missing\n')
-                            % (cset, filename))
+                            _(b'changeset %s: %s missing\n') % (cset, filename)
+                        )
                         failed = True
                     else:
-                        raise RuntimeError('verify failed: unexpected response '
-                                           'from statlfile (%r)' % stat)
+                        raise RuntimeError(
+                            b'verify failed: unexpected response '
+                            b'from statlfile (%r)' % stat
+                        )
         return failed
 
     def _put(self, hash, fd):
         '''Put file with the given hash in the remote store.'''
-        raise NotImplementedError('abstract method')
+        raise NotImplementedError(b'abstract method')
 
     def _get(self, hash):
         '''Get a iterator for content with the given hash.'''
-        raise NotImplementedError('abstract method')
+        raise NotImplementedError(b'abstract method')
 
     def _stat(self, hashes):
         '''Get information about availability of files specified by
         hashes in the remote store. Return dictionary mapping hashes
         to return code where 0 means that file is available, other
         values if not.'''
-        raise NotImplementedError('abstract method')
+        raise NotImplementedError(b'abstract method')
