@@ -11,13 +11,14 @@ from . import (
     remotestore,
 )
 
+
 class wirestore(remotestore.remotestore):
     def __init__(self, ui, repo, remote):
-        cap = remote.capable('largefiles')
+        cap = remote.capable(b'largefiles')
         if not cap:
             raise lfutil.storeprotonotcapable([])
-        storetypes = cap.split(',')
-        if 'serve' not in storetypes:
+        storetypes = cap.split(b',')
+        if b'serve' not in storetypes:
             raise lfutil.storeprotonotcapable(storetypes)
         self.remote = remote
         super(wirestore, self).__init__(ui, repo, remote.url())
@@ -32,8 +33,10 @@ class wirestore(remotestore.remotestore):
         '''For each hash, return 0 if it is available, other values if not.
         It is usually 2 if the largefile is missing, but might be 1 the server
         has a corrupted copy.'''
-        batch = self.remote.iterbatch()
-        for hash in hashes:
-            batch.statlfile(hash)
-        batch.submit()
-        return dict(zip(hashes, batch.results()))
+
+        with self.remote.commandexecutor() as e:
+            fs = []
+            for hash in hashes:
+                fs.append((hash, e.callcommand(b'statlfile', {b'sha': hash,})))
+
+            return {hash: f.result() for hash, f in fs}
