@@ -11,14 +11,31 @@ import hashlib
 
 from mercurial.i18n import _
 from mercurial.node import bin, hex, nullid, short
+from mercurial.pycompat import (
+    getattr,
+    setattr,
+)
 
 from mercurial import (
+    bundle2,
+    changegroup,
+    cmdutil,
+    context,
     error,
-    repository,
+    exchange,
+    exthelper,
+    localrepo,
+    pycompat,
     revlog,
+    scmutil,
+    upgrade,
     util,
+    vfs as vfsmod,
+    wireprotov1server,
 )
 
+from mercurial.interfaces import repository
+
 from mercurial.utils import (
     storageutil,
     stringutil,
@@ -31,34 +48,44 @@ from . import (
     pointer,
 )
 
+eh = exthelper.exthelper()
+
+
+@eh.wrapfunction(localrepo, b'makefilestorage')
 def localrepomakefilestorage(orig, requirements, features, **kwargs):
     if b'lfs' in requirements:
         features.add(repository.REPO_FEATURE_LFS)
 
     return orig(requirements=requirements, features=features, **kwargs)
 
+
+@eh.wrapfunction(changegroup, b'allsupportedversions')
 def allsupportedversions(orig, ui):
     versions = orig(ui)
-    versions.add('03')
+    versions.add(b'03')
     return versions
 
+
+@eh.wrapfunction(wireprotov1server, b'_capabilities')
 def _capabilities(orig, repo, proto):
     '''Wrap server command to announce lfs server capability'''
     caps = orig(repo, proto)
-    if util.safehasattr(repo.svfs, 'lfslocalblobstore'):
+    if util.safehasattr(repo.svfs, b'lfslocalblobstore'):
         # Advertise a slightly different capability when lfs is *required*, so
         # that the client knows it MUST load the extension.  If lfs is not
         # required on the server, there's no reason to autoload the extension
         # on the client.
         if b'lfs' in repo.requirements:
-            caps.append('lfs-serve')
+            caps.append(b'lfs-serve')
 
-        caps.append('lfs')
+        caps.append(b'lfs')
     return caps
 
+
 def bypasscheckhash(self, text):
     return False
 
+
 def readfromstore(self, text):
     """Read filelog content from local blobstore transform for flagprocessor.
 
@@ -79,15 +106,16 @@ def readfromstore(self, text):
     # pack hg filelog metadata
     hgmeta = {}
     for k in p.keys():
-        if k.startswith('x-hg-'):
-            name = k[len('x-hg-'):]
+        if k.startswith(b'x-hg-'):
+            name = k[len(b'x-hg-') :]
             hgmeta[name] = p[k]
-    if hgmeta or text.startswith('\1\n'):
+    if hgmeta or text.startswith(b'\1\n'):
         text = storageutil.packmeta(hgmeta, text)
 
-    return (text, True)
+    return (text, True, {})
 
-def writetostore(self, text):
+
+def writetostore(self, text, sidedata):
     # hg filelog metadata (includes rename, etc)
     hgmeta, offset = storageutil.parsemeta(text)
     if offset and offset > 0:
@@ -99,42 +127,55 @@ def writetostore(self, text):
     self.opener.lfslocalblobstore.write(oid, text)
 
     # replace contents with metadata
-    longoid = 'sha256:%s' % oid
-    metadata = pointer.gitlfspointer(oid=longoid, size='%d' % len(text))
+    longoid = b'sha256:%s' % oid
+    metadata = pointer.gitlfspointer(oid=longoid, size=b'%d' % len(text))
 
     # by default, we expect the content to be binary. however, LFS could also
     # be used for non-binary content. add a special entry for non-binary data.
     # this will be used by filectx.isbinary().
     if not stringutil.binary(text):
         # not hg filelog metadata (affecting commit hash), no "x-hg-" prefix
-        metadata['x-is-binary'] = '0'
+        metadata[b'x-is-binary'] = b'0'
 
     # translate hg filelog metadata to lfs metadata with "x-hg-" prefix
     if hgmeta is not None:
-        for k, v in hgmeta.iteritems():
-            metadata['x-hg-%s' % k] = v
+        for k, v in pycompat.iteritems(hgmeta):
+            metadata[b'x-hg-%s' % k] = v
 
     rawtext = metadata.serialize()
     return (rawtext, False)
 
+
 def _islfs(rlog, node=None, rev=None):
     if rev is None:
         if node is None:
             # both None - likely working copy content where node is not ready
             return False
-        rev = rlog._revlog.rev(node)
+        rev = rlog.rev(node)
     else:
-        node = rlog._revlog.node(rev)
+        node = rlog.node(rev)
     if node == nullid:
         return False
-    flags = rlog._revlog.flags(rev)
+    flags = rlog.flags(rev)
     return bool(flags & revlog.REVIDX_EXTSTORED)
 
-def filelogaddrevision(orig, self, text, transaction, link, p1, p2,
-                       cachedelta=None, node=None,
-                       flags=revlog.REVIDX_DEFAULT_FLAGS, **kwds):
+
+# Wrapping may also be applied by remotefilelog
+def filelogaddrevision(
+    orig,
+    self,
+    text,
+    transaction,
+    link,
+    p1,
+    p2,
+    cachedelta=None,
+    node=None,
+    flags=revlog.REVIDX_DEFAULT_FLAGS,
+    **kwds
+):
     # The matcher isn't available if reposetup() wasn't called.
-    lfstrack = self._revlog.opener.options.get('lfstrack')
+    lfstrack = self._revlog.opener.options.get(b'lfstrack')
 
     if lfstrack:
         textlen = len(text)
@@ -146,29 +187,64 @@ def filelogaddrevision(orig, self, text, transaction, link, p1, p2,
         if lfstrack(self._revlog.filename, textlen):
             flags |= revlog.REVIDX_EXTSTORED
 
-    return orig(self, text, transaction, link, p1, p2, cachedelta=cachedelta,
-                node=node, flags=flags, **kwds)
+    return orig(
+        self,
+        text,
+        transaction,
+        link,
+        p1,
+        p2,
+        cachedelta=cachedelta,
+        node=node,
+        flags=flags,
+        **kwds
+    )
 
+
+# Wrapping may also be applied by remotefilelog
 def filelogrenamed(orig, self, node):
-    if _islfs(self, node):
-        rawtext = self._revlog.revision(node, raw=True)
+    if _islfs(self._revlog, node):
+        rawtext = self._revlog.rawdata(node)
         if not rawtext:
             return False
         metadata = pointer.deserialize(rawtext)
-        if 'x-hg-copy' in metadata and 'x-hg-copyrev' in metadata:
-            return metadata['x-hg-copy'], bin(metadata['x-hg-copyrev'])
+        if b'x-hg-copy' in metadata and b'x-hg-copyrev' in metadata:
+            return metadata[b'x-hg-copy'], bin(metadata[b'x-hg-copyrev'])
         else:
             return False
     return orig(self, node)
 
+
+# Wrapping may also be applied by remotefilelog
 def filelogsize(orig, self, rev):
-    if _islfs(self, rev=rev):
+    if _islfs(self._revlog, rev=rev):
         # fast path: use lfs metadata to answer size
-        rawtext = self._revlog.revision(rev, raw=True)
+        rawtext = self._revlog.rawdata(rev)
         metadata = pointer.deserialize(rawtext)
-        return int(metadata['size'])
+        return int(metadata[b'size'])
     return orig(self, rev)
 
+
+@eh.wrapfunction(revlog, b'_verify_revision')
+def _verify_revision(orig, rl, skipflags, state, node):
+    if _islfs(rl, node=node):
+        rawtext = rl.rawdata(node)
+        metadata = pointer.deserialize(rawtext)
+
+        # Don't skip blobs that are stored locally, as local verification is
+        # relatively cheap and there's no other way to verify the raw data in
+        # the revlog.
+        if rl.opener.lfslocalblobstore.has(metadata.oid()):
+            skipflags &= ~revlog.REVIDX_EXTSTORED
+        elif skipflags & revlog.REVIDX_EXTSTORED:
+            # The wrapped method will set `skipread`, but there's enough local
+            # info to check renames.
+            state[b'safe_renamed'].add(node)
+
+    orig(rl, skipflags, state, node)
+
+
+@eh.wrapfunction(context.basefilectx, b'cmp')
 def filectxcmp(orig, self, fctx):
     """returns True if text is different than fctx"""
     # some fctx (ex. hg-git) is not based on basefilectx and do not have islfs
@@ -179,38 +255,63 @@ def filectxcmp(orig, self, fctx):
         return p1.oid() != p2.oid()
     return orig(self, fctx)
 
+
+@eh.wrapfunction(context.basefilectx, b'isbinary')
 def filectxisbinary(orig, self):
     if self.islfs():
         # fast path: use lfs metadata to answer isbinary
         metadata = pointer.deserialize(self.rawdata())
         # if lfs metadata says nothing, assume it's binary by default
-        return bool(int(metadata.get('x-is-binary', 1)))
+        return bool(int(metadata.get(b'x-is-binary', 1)))
     return orig(self)
 
-def filectxislfs(self):
-    return _islfs(self.filelog(), self.filenode())
 
+def filectxislfs(self):
+    return _islfs(self.filelog()._revlog, self.filenode())
+
+
+@eh.wrapfunction(cmdutil, b'_updatecatformatter')
 def _updatecatformatter(orig, fm, ctx, matcher, path, decode):
     orig(fm, ctx, matcher, path, decode)
     fm.data(rawdata=ctx[path].rawdata())
 
+
+@eh.wrapfunction(scmutil, b'wrapconvertsink')
 def convertsink(orig, sink):
     sink = orig(sink)
-    if sink.repotype == 'hg':
-        class lfssink(sink.__class__):
-            def putcommit(self, files, copies, parents, commit, source, revmap,
-                          full, cleanp2):
-                pc = super(lfssink, self).putcommit
-                node = pc(files, copies, parents, commit, source, revmap, full,
-                          cleanp2)
+    if sink.repotype == b'hg':
 
-                if 'lfs' not in self.repo.requirements:
+        class lfssink(sink.__class__):
+            def putcommit(
+                self,
+                files,
+                copies,
+                parents,
+                commit,
+                source,
+                revmap,
+                full,
+                cleanp2,
+            ):
+                pc = super(lfssink, self).putcommit
+                node = pc(
+                    files,
+                    copies,
+                    parents,
+                    commit,
+                    source,
+                    revmap,
+                    full,
+                    cleanp2,
+                )
+
+                if b'lfs' not in self.repo.requirements:
                     ctx = self.repo[node]
 
                     # The file list may contain removed files, so check for
                     # membership before assuming it is in the context.
                     if any(f in ctx and ctx[f].islfs() for f, n in files):
-                        self.repo.requirements.add('lfs')
+                        self.repo.requirements.add(b'lfs')
                         self.repo._writerequirements()
 
                 return node
@@ -219,22 +320,27 @@ def convertsink(orig, sink):
 
     return sink
 
+
+# bundlerepo uses "vfsmod.readonlyvfs(othervfs)", we need to make sure lfs
+# options and blob stores are passed from othervfs to the new readonlyvfs.
+@eh.wrapfunction(vfsmod.readonlyvfs, b'__init__')
 def vfsinit(orig, self, othervfs):
     orig(self, othervfs)
     # copy lfs related options
     for k, v in othervfs.options.items():
-        if k.startswith('lfs'):
+        if k.startswith(b'lfs'):
             self.options[k] = v
     # also copy lfs blobstores. note: this can run before reposetup, so lfs
     # blobstore attributes are not always ready at this time.
-    for name in ['lfslocalblobstore', 'lfsremoteblobstore']:
+    for name in [b'lfslocalblobstore', b'lfsremoteblobstore']:
         if util.safehasattr(othervfs, name):
             setattr(self, name, getattr(othervfs, name))
 
+
 def _prefetchfiles(repo, revs, match):
     """Ensure that required LFS blobs are present, fetching them as a group if
     needed."""
-    if not util.safehasattr(repo.svfs, 'lfslocalblobstore'):
+    if not util.safehasattr(repo.svfs, b'lfslocalblobstore'):
         return
 
     pointers = []
@@ -255,22 +361,25 @@ def _prefetchfiles(repo, revs, match):
         # on the repo by a clone command to be used for the update.
         blobstore.remote(repo).readbatch(pointers, localstore)
 
+
 def _canskipupload(repo):
     # Skip if this hasn't been passed to reposetup()
-    if not util.safehasattr(repo.svfs, 'lfsremoteblobstore'):
+    if not util.safehasattr(repo.svfs, b'lfsremoteblobstore'):
         return True
 
     # if remotestore is a null store, upload is a no-op and can be skipped
     return isinstance(repo.svfs.lfsremoteblobstore, blobstore._nullremote)
 
+
 def candownload(repo):
     # Skip if this hasn't been passed to reposetup()
-    if not util.safehasattr(repo.svfs, 'lfsremoteblobstore'):
+    if not util.safehasattr(repo.svfs, b'lfsremoteblobstore'):
         return False
 
     # if remotestore is a null store, downloads will lead to nothing
     return not isinstance(repo.svfs.lfsremoteblobstore, blobstore._nullremote)
 
+
 def uploadblobsfromrevs(repo, revs):
     '''upload lfs blobs introduced by revs
 
@@ -281,6 +390,7 @@ def uploadblobsfromrevs(repo, revs):
     pointers = extractpointers(repo, revs)
     uploadblobs(repo, pointers)
 
+
 def prepush(pushop):
     """Prepush hook.
 
@@ -290,19 +400,22 @@ def prepush(pushop):
     """
     return uploadblobsfromrevs(pushop.repo, pushop.outgoing.missing)
 
+
+@eh.wrapfunction(exchange, b'push')
 def push(orig, repo, remote, *args, **kwargs):
     """bail on push if the extension isn't enabled on remote when needed, and
     update the remote store based on the destination path."""
-    if 'lfs' in repo.requirements:
+    if b'lfs' in repo.requirements:
         # If the remote peer is for a local repo, the requirement tests in the
         # base class method enforce lfs support.  Otherwise, some revisions in
         # this repo use lfs, and the remote repo needs the extension loaded.
-        if not remote.local() and not remote.capable('lfs'):
+        if not remote.local() and not remote.capable(b'lfs'):
             # This is a copy of the message in exchange.push() when requirements
             # are missing between local repos.
-            m = _("required features are not supported in the destination: %s")
-            raise error.Abort(m % 'lfs',
-                              hint=_('enable the lfs extension on the server'))
+            m = _(b"required features are not supported in the destination: %s")
+            raise error.Abort(
+                m % b'lfs', hint=_(b'enable the lfs extension on the server')
+            )
 
         # Repositories where this extension is disabled won't have the field.
         # But if there's a requirement, then the extension must be loaded AND
@@ -316,20 +429,28 @@ def push(orig, repo, remote, *args, **kwargs):
     else:
         return orig(repo, remote, *args, **kwargs)
 
-def writenewbundle(orig, ui, repo, source, filename, bundletype, outgoing,
-                   *args, **kwargs):
+
+# when writing a bundle via "hg bundle" command, upload related LFS blobs
+@eh.wrapfunction(bundle2, b'writenewbundle')
+def writenewbundle(
+    orig, ui, repo, source, filename, bundletype, outgoing, *args, **kwargs
+):
     """upload LFS blobs added by outgoing revisions on 'hg bundle'"""
     uploadblobsfromrevs(repo, outgoing.missing)
-    return orig(ui, repo, source, filename, bundletype, outgoing, *args,
-                **kwargs)
+    return orig(
+        ui, repo, source, filename, bundletype, outgoing, *args, **kwargs
+    )
+
 
 def extractpointers(repo, revs):
     """return a list of lfs pointers added by given revs"""
-    repo.ui.debug('lfs: computing set of blobs to upload\n')
+    repo.ui.debug(b'lfs: computing set of blobs to upload\n')
     pointers = {}
 
     makeprogress = repo.ui.makeprogress
-    with makeprogress(_('lfs search'), _('changesets'), len(revs)) as progress:
+    with makeprogress(
+        _(b'lfs search'), _(b'changesets'), len(revs)
+    ) as progress:
         for r in revs:
             ctx = repo[r]
             for p in pointersfromctx(ctx).values():
@@ -337,6 +458,7 @@ def extractpointers(repo, revs):
             progress.increment()
         return sorted(pointers.values(), key=lambda p: p.oid())
 
+
 def pointerfromctx(ctx, f, removed=False):
     """return a pointer for the named file from the given changectx, or None if
     the file isn't LFS.
@@ -356,7 +478,7 @@ def pointerfromctx(ctx, f, removed=False):
         else:
             return None
     fctx = _ctx[f]
-    if not _islfs(fctx.filelog(), fctx.filenode()):
+    if not _islfs(fctx.filelog()._revlog, fctx.filenode()):
         return None
     try:
         p = pointer.deserialize(fctx.rawdata())
@@ -364,8 +486,11 @@ def pointerfromctx(ctx, f, removed=False):
             return p
         return {}
     except pointer.InvalidPointer as ex:
-        raise error.Abort(_('lfs: corrupted pointer (%s@%s): %s\n')
-                          % (f, short(_ctx.node()), ex))
+        raise error.Abort(
+            _(b'lfs: corrupted pointer (%s@%s): %s\n')
+            % (f, short(_ctx.node()), ex)
+        )
+
 
 def pointersfromctx(ctx, removed=False):
     """return a dict {path: pointer} for given single changectx.
@@ -385,6 +510,7 @@ def pointersfromctx(ctx, removed=False):
             result[f] = p
     return result
 
+
 def uploadblobs(repo, pointers):
     """upload given pointers from local blobstore"""
     if not pointers:
@@ -393,22 +519,28 @@ def uploadblobs(repo, pointers):
     remoteblob = repo.svfs.lfsremoteblobstore
     remoteblob.writebatch(pointers, repo.svfs.lfslocalblobstore)
 
+
+@eh.wrapfunction(upgrade, b'_finishdatamigration')
 def upgradefinishdatamigration(orig, ui, srcrepo, dstrepo, requirements):
     orig(ui, srcrepo, dstrepo, requirements)
 
     # Skip if this hasn't been passed to reposetup()
-    if (util.safehasattr(srcrepo.svfs, 'lfslocalblobstore') and
-        util.safehasattr(dstrepo.svfs, 'lfslocalblobstore')):
+    if util.safehasattr(
+        srcrepo.svfs, b'lfslocalblobstore'
+    ) and util.safehasattr(dstrepo.svfs, b'lfslocalblobstore'):
         srclfsvfs = srcrepo.svfs.lfslocalblobstore.vfs
         dstlfsvfs = dstrepo.svfs.lfslocalblobstore.vfs
 
         for dirpath, dirs, files in srclfsvfs.walk():
             for oid in files:
-                ui.write(_('copying lfs blob %s\n') % oid)
+                ui.write(_(b'copying lfs blob %s\n') % oid)
                 lfutil.link(srclfsvfs.join(oid), dstlfsvfs.join(oid))
 
+
+@eh.wrapfunction(upgrade, b'preservedrequirements')
+@eh.wrapfunction(upgrade, b'supporteddestrequirements')
 def upgraderequirements(orig, repo):
     reqs = orig(repo)
-    if 'lfs' in repo.requirements:
-        reqs.add('lfs')
+    if b'lfs' in repo.requirements:
+        reqs.add(b'lfs')
     return reqs
