@@ -82,6 +82,12 @@ notify.strip
 
 notify.domain
   Default email domain for sender or recipients with no explicit domain.
+  It is also used for the domain part of the ``Message-Id`` when using
+  ``notify.messageidseed``.
+
+notify.messageidseed
+  Create deterministic ``Message-Id`` headers for the mails based on the seed
+  and the revision identifier of the first commit in the changeset.
 
 notify.style
   Style file to use when formatting emails.
@@ -113,6 +119,9 @@ notify.maxsubject
 notify.diffstat
   Set to True to include a diffstat before diff content. Default: True.
 
+notify.showfunc
+  If set, override ``diff.showfunc`` for the diff content. Default: None.
+
 notify.merge
   If True, send notifications for merge changesets. Default: True.
 
@@ -138,18 +147,21 @@ web.baseurl
 '''
 from __future__ import absolute_import
 
-import email
-import email.parser as emailparser
+import email.errors as emailerrors
+import email.utils as emailutils
 import fnmatch
+import hashlib
 import socket
 import time
 
 from mercurial.i18n import _
 from mercurial import (
+    encoding,
     error,
     logcmdutil,
     mail,
     patch,
+    pycompat,
     registrar,
     util,
 )
@@ -162,65 +174,71 @@ from mercurial.utils import (
 # extensions which SHIP WITH MERCURIAL. Non-mainline extensions should
 # be specifying the version(s) of Mercurial they are tested with, or
 # leave the attribute unspecified.
-testedwith = 'ships-with-hg-core'
+testedwith = b'ships-with-hg-core'
 
 configtable = {}
 configitem = registrar.configitem(configtable)
 
-configitem('notify', 'changegroup',
-    default=None,
+configitem(
+    b'notify', b'changegroup', default=None,
 )
-configitem('notify', 'config',
-    default=None,
+configitem(
+    b'notify', b'config', default=None,
 )
-configitem('notify', 'diffstat',
-    default=True,
+configitem(
+    b'notify', b'diffstat', default=True,
 )
-configitem('notify', 'domain',
-    default=None,
+configitem(
+    b'notify', b'domain', default=None,
 )
-configitem('notify', 'fromauthor',
-    default=None,
+configitem(
+    b'notify', b'messageidseed', default=None,
 )
-configitem('notify', 'incoming',
-    default=None,
+configitem(
+    b'notify', b'fromauthor', default=None,
 )
-configitem('notify', 'maxdiff',
-    default=300,
+configitem(
+    b'notify', b'incoming', default=None,
 )
-configitem('notify', 'maxdiffstat',
-    default=-1,
+configitem(
+    b'notify', b'maxdiff', default=300,
 )
-configitem('notify', 'maxsubject',
-    default=67,
+configitem(
+    b'notify', b'maxdiffstat', default=-1,
 )
-configitem('notify', 'mbox',
-    default=None,
+configitem(
+    b'notify', b'maxsubject', default=67,
 )
-configitem('notify', 'merge',
-    default=True,
+configitem(
+    b'notify', b'mbox', default=None,
 )
-configitem('notify', 'outgoing',
-    default=None,
+configitem(
+    b'notify', b'merge', default=True,
 )
-configitem('notify', 'sources',
-    default='serve',
+configitem(
+    b'notify', b'outgoing', default=None,
 )
-configitem('notify', 'strip',
-    default=0,
+configitem(
+    b'notify', b'sources', default=b'serve',
 )
-configitem('notify', 'style',
-    default=None,
+configitem(
+    b'notify', b'showfunc', default=None,
 )
-configitem('notify', 'template',
-    default=None,
+configitem(
+    b'notify', b'strip', default=0,
 )
-configitem('notify', 'test',
-    default=True,
+configitem(
+    b'notify', b'style', default=None,
+)
+configitem(
+    b'notify', b'template', default=None,
+)
+configitem(
+    b'notify', b'test', default=True,
 )
 
 # template for single changeset can include email headers.
-single_template = '''
+single_template = b'''
 Subject: changeset in {webroot}: {desc|firstline|strip}
 From: {author}
 
@@ -233,39 +251,45 @@ description:
 # template for multiple changesets should not contain email headers,
 # because only first set of headers will be used and result will look
 # strange.
-multiple_template = '''
+multiple_template = b'''
 changeset {node|short} in {root}
 details: {baseurl}{webroot}?cmd=changeset;node={node|short}
 summary: {desc|firstline}
 '''
 
 deftemplates = {
-    'changegroup': multiple_template,
+    b'changegroup': multiple_template,
 }
 
+
 class notifier(object):
     '''email notification class.'''
 
     def __init__(self, ui, repo, hooktype):
         self.ui = ui
-        cfg = self.ui.config('notify', 'config')
+        cfg = self.ui.config(b'notify', b'config')
         if cfg:
-            self.ui.readconfig(cfg, sections=['usersubs', 'reposubs'])
+            self.ui.readconfig(cfg, sections=[b'usersubs', b'reposubs'])
         self.repo = repo
-        self.stripcount = int(self.ui.config('notify', 'strip'))
+        self.stripcount = int(self.ui.config(b'notify', b'strip'))
         self.root = self.strip(self.repo.root)
-        self.domain = self.ui.config('notify', 'domain')
-        self.mbox = self.ui.config('notify', 'mbox')
-        self.test = self.ui.configbool('notify', 'test')
+        self.domain = self.ui.config(b'notify', b'domain')
+        self.mbox = self.ui.config(b'notify', b'mbox')
+        self.test = self.ui.configbool(b'notify', b'test')
         self.charsets = mail._charsets(self.ui)
         self.subs = self.subscribers()
-        self.merge = self.ui.configbool('notify', 'merge')
+        self.merge = self.ui.configbool(b'notify', b'merge')
+        self.showfunc = self.ui.configbool(b'notify', b'showfunc')
+        self.messageidseed = self.ui.config(b'notify', b'messageidseed')
+        if self.showfunc is None:
+            self.showfunc = self.ui.configbool(b'diff', b'showfunc')
 
         mapfile = None
-        template = (self.ui.config('notify', hooktype) or
-                    self.ui.config('notify', 'template'))
+        template = self.ui.config(b'notify', hooktype) or self.ui.config(
+            b'notify', b'template'
+        )
         if not template:
-            mapfile = self.ui.config('notify', 'style')
+            mapfile = self.ui.config(b'notify', b'style')
         if not mapfile and not template:
             template = deftemplates.get(hooktype) or single_template
         spec = logcmdutil.templatespec(template, mapfile)
@@ -277,10 +301,10 @@ class notifier(object):
         path = util.pconvert(path)
         count = self.stripcount
         while count > 0:
-            c = path.find('/')
+            c = path.find(b'/')
             if c == -1:
                 break
-            path = path[c + 1:]
+            path = path[c + 1 :]
             count -= 1
         return path
 
@@ -289,47 +313,54 @@ class notifier(object):
 
         addr = stringutil.email(addr.strip())
         if self.domain:
-            a = addr.find('@localhost')
+            a = addr.find(b'@localhost')
             if a != -1:
                 addr = addr[:a]
-            if '@' not in addr:
-                return addr + '@' + self.domain
+            if b'@' not in addr:
+                return addr + b'@' + self.domain
         return addr
 
     def subscribers(self):
         '''return list of email addresses of subscribers to this repo.'''
         subs = set()
-        for user, pats in self.ui.configitems('usersubs'):
-            for pat in pats.split(','):
-                if '#' in pat:
-                    pat, revs = pat.split('#', 1)
+        for user, pats in self.ui.configitems(b'usersubs'):
+            for pat in pats.split(b','):
+                if b'#' in pat:
+                    pat, revs = pat.split(b'#', 1)
                 else:
                     revs = None
                 if fnmatch.fnmatch(self.repo.root, pat.strip()):
                     subs.add((self.fixmail(user), revs))
-        for pat, users in self.ui.configitems('reposubs'):
-            if '#' in pat:
-                pat, revs = pat.split('#', 1)
+        for pat, users in self.ui.configitems(b'reposubs'):
+            if b'#' in pat:
+                pat, revs = pat.split(b'#', 1)
             else:
                 revs = None
             if fnmatch.fnmatch(self.repo.root, pat):
-                for user in users.split(','):
+                for user in users.split(b','):
                     subs.add((self.fixmail(user), revs))
-        return [(mail.addressencode(self.ui, s, self.charsets, self.test), r)
-                for s, r in sorted(subs)]
+        return [
+            (mail.addressencode(self.ui, s, self.charsets, self.test), r)
+            for s, r in sorted(subs)
+        ]
 
     def node(self, ctx, **props):
         '''format one changeset, unless it is a suppressed merge.'''
         if not self.merge and len(ctx.parents()) > 1:
             return False
-        self.t.show(ctx, changes=ctx.changeset(),
-                    baseurl=self.ui.config('web', 'baseurl'),
-                    root=self.repo.root, webroot=self.root, **props)
+        self.t.show(
+            ctx,
+            changes=ctx.changeset(),
+            baseurl=self.ui.config(b'web', b'baseurl'),
+            root=self.repo.root,
+            webroot=self.root,
+            **props
+        )
         return True
 
     def skipsource(self, source):
         '''true if incoming changes from this source should be skipped.'''
-        ok_sources = self.ui.config('notify', 'sources').split()
+        ok_sources = self.ui.config(b'notify', b'sources').split()
         return source not in ok_sources
 
     def send(self, ctx, count, data):
@@ -341,112 +372,128 @@ class notifier(object):
             if spec is None:
                 subs.add(sub)
                 continue
-            revs = self.repo.revs('%r and %d:', spec, ctx.rev())
+            revs = self.repo.revs(b'%r and %d:', spec, ctx.rev())
             if len(revs):
                 subs.add(sub)
                 continue
         if len(subs) == 0:
-            self.ui.debug('notify: no subscribers to selected repo '
-                          'and revset\n')
+            self.ui.debug(
+                b'notify: no subscribers to selected repo and revset\n'
+            )
             return
 
-        p = emailparser.Parser()
         try:
-            msg = p.parsestr(data)
-        except email.Errors.MessageParseError as inst:
+            msg = mail.parsebytes(data)
+        except emailerrors.MessageParseError as inst:
             raise error.Abort(inst)
 
         # store sender and subject
-        sender, subject = msg['From'], msg['Subject']
+        sender = msg['From']
+        subject = msg['Subject']
+        if sender is not None:
+            sender = mail.headdecode(sender)
+        if subject is not None:
+            subject = mail.headdecode(subject)
         del msg['From'], msg['Subject']
 
         if not msg.is_multipart():
             # create fresh mime message from scratch
             # (multipart templates must take care of this themselves)
             headers = msg.items()
-            payload = msg.get_payload()
+            payload = msg.get_payload(decode=pycompat.ispy3)
             # for notification prefer readability over data precision
             msg = mail.mimeencode(self.ui, payload, self.charsets, self.test)
             # reinstate custom headers
             for k, v in headers:
                 msg[k] = v
 
-        msg['Date'] = dateutil.datestr(format="%a, %d %b %Y %H:%M:%S %1%2")
+        msg['Date'] = encoding.strfromlocal(
+            dateutil.datestr(format=b"%a, %d %b %Y %H:%M:%S %1%2")
+        )
 
         # try to make subject line exist and be useful
         if not subject:
             if count > 1:
-                subject = _('%s: %d new changesets') % (self.root, count)
+                subject = _(b'%s: %d new changesets') % (self.root, count)
             else:
-                s = ctx.description().lstrip().split('\n', 1)[0].rstrip()
-                subject = '%s: %s' % (self.root, s)
-        maxsubject = int(self.ui.config('notify', 'maxsubject'))
+                s = ctx.description().lstrip().split(b'\n', 1)[0].rstrip()
+                subject = b'%s: %s' % (self.root, s)
+        maxsubject = int(self.ui.config(b'notify', b'maxsubject'))
         if maxsubject:
             subject = stringutil.ellipsis(subject, maxsubject)
-        msg['Subject'] = mail.headencode(self.ui, subject,
-                                         self.charsets, self.test)
+        msg['Subject'] = mail.headencode(
+            self.ui, subject, self.charsets, self.test
+        )
 
         # try to make message have proper sender
         if not sender:
-            sender = self.ui.config('email', 'from') or self.ui.username()
-        if '@' not in sender or '@localhost' in sender:
+            sender = self.ui.config(b'email', b'from') or self.ui.username()
+        if b'@' not in sender or b'@localhost' in sender:
             sender = self.fixmail(sender)
-        msg['From'] = mail.addressencode(self.ui, sender,
-                                         self.charsets, self.test)
+        msg['From'] = mail.addressencode(
+            self.ui, sender, self.charsets, self.test
+        )
 
         msg['X-Hg-Notification'] = 'changeset %s' % ctx
         if not msg['Message-Id']:
-            msg['Message-Id'] = ('<hg.%s.%s.%s@%s>' %
-                                 (ctx, int(time.time()),
-                                  hash(self.repo.root), socket.getfqdn()))
+            msg['Message-Id'] = messageid(ctx, self.domain, self.messageidseed)
         msg['To'] = ', '.join(sorted(subs))
 
-        msgtext = msg.as_string()
+        msgtext = msg.as_bytes() if pycompat.ispy3 else msg.as_string()
         if self.test:
             self.ui.write(msgtext)
-            if not msgtext.endswith('\n'):
-                self.ui.write('\n')
+            if not msgtext.endswith(b'\n'):
+                self.ui.write(b'\n')
         else:
-            self.ui.status(_('notify: sending %d subscribers %d changes\n') %
-                           (len(subs), count))
-            mail.sendmail(self.ui, stringutil.email(msg['From']),
-                          subs, msgtext, mbox=self.mbox)
+            self.ui.status(
+                _(b'notify: sending %d subscribers %d changes\n')
+                % (len(subs), count)
+            )
+            mail.sendmail(
+                self.ui,
+                emailutils.parseaddr(msg['From'])[1],
+                subs,
+                msgtext,
+                mbox=self.mbox,
+            )
 
     def diff(self, ctx, ref=None):
 
-        maxdiff = int(self.ui.config('notify', 'maxdiff'))
+        maxdiff = int(self.ui.config(b'notify', b'maxdiff'))
         prev = ctx.p1().node()
         if ref:
             ref = ref.node()
         else:
             ref = ctx.node()
-        chunks = patch.diff(self.repo, prev, ref,
-                            opts=patch.diffallopts(self.ui))
-        difflines = ''.join(chunks).splitlines()
+        diffopts = patch.diffallopts(self.ui)
+        diffopts.showfunc = self.showfunc
+        chunks = patch.diff(self.repo, prev, ref, opts=diffopts)
+        difflines = b''.join(chunks).splitlines()
 
-        if self.ui.configbool('notify', 'diffstat'):
-            maxdiffstat = int(self.ui.config('notify', 'maxdiffstat'))
+        if self.ui.configbool(b'notify', b'diffstat'):
+            maxdiffstat = int(self.ui.config(b'notify', b'maxdiffstat'))
             s = patch.diffstat(difflines)
             # s may be nil, don't include the header if it is
             if s:
-                if maxdiffstat >= 0 and s.count("\n") > maxdiffstat + 1:
-                    s = s.split("\n")
-                    msg = _('\ndiffstat (truncated from %d to %d lines):\n\n')
+                if maxdiffstat >= 0 and s.count(b"\n") > maxdiffstat + 1:
+                    s = s.split(b"\n")
+                    msg = _(b'\ndiffstat (truncated from %d to %d lines):\n\n')
                     self.ui.write(msg % (len(s) - 2, maxdiffstat))
-                    self.ui.write("\n".join(s[:maxdiffstat] + s[-2:]))
+                    self.ui.write(b"\n".join(s[:maxdiffstat] + s[-2:]))
                 else:
-                    self.ui.write(_('\ndiffstat:\n\n%s') % s)
+                    self.ui.write(_(b'\ndiffstat:\n\n%s') % s)
 
         if maxdiff == 0:
             return
         elif maxdiff > 0 and len(difflines) > maxdiff:
-            msg = _('\ndiffs (truncated from %d to %d lines):\n\n')
+            msg = _(b'\ndiffs (truncated from %d to %d lines):\n\n')
             self.ui.write(msg % (len(difflines), maxdiff))
             difflines = difflines[:maxdiff]
         elif difflines:
-            self.ui.write(_('\ndiffs (%d lines):\n\n') % len(difflines))
+            self.ui.write(_(b'\ndiffs (%d lines):\n\n') % len(difflines))
+
+        self.ui.write(b"\n".join(difflines))
 
-        self.ui.write("\n".join(difflines))
 
 def hook(ui, repo, hooktype, node=None, source=None, **kwargs):
     '''send email notifications to interested subscribers.
@@ -458,17 +505,17 @@ def hook(ui, repo, hooktype, node=None, source=None, **kwargs):
     ctx = repo.unfiltered()[node]
 
     if not n.subs:
-        ui.debug('notify: no subscribers to repository %s\n' % n.root)
+        ui.debug(b'notify: no subscribers to repository %s\n' % n.root)
         return
     if n.skipsource(source):
-        ui.debug('notify: changes have source "%s" - skipping\n' % source)
+        ui.debug(b'notify: changes have source "%s" - skipping\n' % source)
         return
 
     ui.pushbuffer()
-    data = ''
+    data = b''
     count = 0
-    author = ''
-    if hooktype == 'changegroup' or hooktype == 'outgoing':
+    author = b''
+    if hooktype == b'changegroup' or hooktype == b'outgoing':
         for rev in repo.changelog.revs(start=ctx.rev()):
             if n.node(repo[rev]):
                 count += 1
@@ -476,16 +523,20 @@ def hook(ui, repo, hooktype, node=None, source=None, **kwargs):
                     author = repo[rev].user()
             else:
                 data += ui.popbuffer()
-                ui.note(_('notify: suppressing notification for merge %d:%s\n')
-                        % (rev, repo[rev].hex()[:12]))
+                ui.note(
+                    _(b'notify: suppressing notification for merge %d:%s\n')
+                    % (rev, repo[rev].hex()[:12])
+                )
                 ui.pushbuffer()
         if count:
-            n.diff(ctx, repo['tip'])
+            n.diff(ctx, repo[b'tip'])
     elif ctx.rev() in repo:
         if not n.node(ctx):
             ui.popbuffer()
-            ui.note(_('notify: suppressing notification for merge %d:%s\n') %
-                    (ctx.rev(), ctx.hex()[:12]))
+            ui.note(
+                _(b'notify: suppressing notification for merge %d:%s\n')
+                % (ctx.rev(), ctx.hex()[:12])
+            )
             return
         count += 1
         n.diff(ctx)
@@ -493,9 +544,30 @@ def hook(ui, repo, hooktype, node=None, source=None, **kwargs):
             author = ctx.user()
 
     data += ui.popbuffer()
-    fromauthor = ui.config('notify', 'fromauthor')
+    fromauthor = ui.config(b'notify', b'fromauthor')
     if author and fromauthor:
-        data = '\n'.join(['From: %s' % author, data])
+        data = b'\n'.join([b'From: %s' % author, data])
 
     if count:
         n.send(ctx, count, data)
+
+
+def messageid(ctx, domain, messageidseed):
+    if domain and messageidseed:
+        host = domain
+    else:
+        host = encoding.strtolocal(socket.getfqdn())
+    if messageidseed:
+        messagehash = hashlib.sha512(ctx.hex() + messageidseed)
+        messageid = b'<hg.%s@%s>' % (
+            pycompat.sysbytes(messagehash.hexdigest()[:64]),
+            host,
+        )
+    else:
+        messageid = b'<hg.%s.%d.%d@%s>' % (
+            ctx,
+            int(time.time()),
+            hash(ctx.repo().root),
+            host,
+        )
+    return encoding.strfromlocal(messageid)
