@@ -16,9 +16,8 @@ import zipfile
 import zlib
 
 from .i18n import _
-from .node import (
-    nullrev,
-)
+from .node import nullrev
+from .pycompat import open
 
 from . import (
     error,
@@ -29,11 +28,13 @@ from . import (
     util,
     vfs as vfsmod,
 )
+
 stringio = util.stringio
 
 # from unzip source code:
 _UNX_IFREG = 0x8000
-_UNX_IFLNK = 0xa000
+_UNX_IFLNK = 0xA000
+
 
 def tidyprefix(dest, kind, prefix):
     '''choose prefix to use for names in archive.  make sure prefix is
@@ -43,44 +44,49 @@ def tidyprefix(dest, kind, prefix):
         prefix = util.normpath(prefix)
     else:
         if not isinstance(dest, bytes):
-            raise ValueError('dest must be string if no prefix')
+            raise ValueError(b'dest must be string if no prefix')
         prefix = os.path.basename(dest)
         lower = prefix.lower()
         for sfx in exts.get(kind, []):
             if lower.endswith(sfx):
-                prefix = prefix[:-len(sfx)]
+                prefix = prefix[: -len(sfx)]
                 break
     lpfx = os.path.normpath(util.localpath(prefix))
     prefix = util.pconvert(lpfx)
-    if not prefix.endswith('/'):
-        prefix += '/'
+    if not prefix.endswith(b'/'):
+        prefix += b'/'
     # Drop the leading '.' path component if present, so Windows can read the
     # zip files (issue4634)
-    if prefix.startswith('./'):
+    if prefix.startswith(b'./'):
         prefix = prefix[2:]
-    if prefix.startswith('../') or os.path.isabs(lpfx) or '/../' in prefix:
-        raise error.Abort(_('archive prefix contains illegal components'))
+    if prefix.startswith(b'../') or os.path.isabs(lpfx) or b'/../' in prefix:
+        raise error.Abort(_(b'archive prefix contains illegal components'))
     return prefix
 
+
 exts = {
-    'tar': ['.tar'],
-    'tbz2': ['.tbz2', '.tar.bz2'],
-    'tgz': ['.tgz', '.tar.gz'],
-    'zip': ['.zip'],
-    }
+    b'tar': [b'.tar'],
+    b'tbz2': [b'.tbz2', b'.tar.bz2'],
+    b'tgz': [b'.tgz', b'.tar.gz'],
+    b'zip': [b'.zip'],
+    b'txz': [b'.txz', b'.tar.xz'],
+}
+
 
 def guesskind(dest):
-    for kind, extensions in exts.iteritems():
+    for kind, extensions in pycompat.iteritems(exts):
         if any(dest.endswith(ext) for ext in extensions):
             return kind
     return None
 
+
 def _rootctx(repo):
     # repo[0] may be hidden
     for rev in repo:
         return repo[rev]
     return repo[nullrev]
 
+
 # {tags} on ctx includes local tags and 'tip', with no current way to limit
 # that to global tags.  Therefore, use {latesttag} as a substitute when
 # the distance is 0, since that will be the list of global tags on ctx.
@@ -93,43 +99,47 @@ branch: {branch|utf8}
                join(latesttag % "latesttag: {tag}", "\n"),
                "latesttagdistance: {latesttagdistance}",
                "changessincelatesttag: {changessincelatesttag}"))}
-'''[1:]  # drop leading '\n'
+'''[
+    1:
+]  # drop leading '\n'
+
 
 def buildmetadata(ctx):
     '''build content of .hg_archival.txt'''
     repo = ctx.repo()
 
     opts = {
-        'template': repo.ui.config('experimental', 'archivemetatemplate',
-                                   _defaultmetatemplate)
+        b'template': repo.ui.config(
+            b'experimental', b'archivemetatemplate', _defaultmetatemplate
+        )
     }
 
     out = util.stringio()
 
-    fm = formatter.formatter(repo.ui, out, 'archive', opts)
+    fm = formatter.formatter(repo.ui, out, b'archive', opts)
     fm.startitem()
     fm.context(ctx=ctx)
     fm.data(root=_rootctx(repo).hex())
 
     if ctx.rev() is None:
-        dirty = ''
+        dirty = b''
         if ctx.dirty(missing=True):
-            dirty = '+'
+            dirty = b'+'
         fm.data(dirty=dirty)
     fm.end()
 
     return out.getvalue()
 
+
 class tarit(object):
     '''write archive to tar file or stream.  can write uncompressed,
     or compress with gzip or bzip2.'''
 
     class GzipFileWithTime(gzip.GzipFile):
-
         def __init__(self, *args, **kw):
             timestamp = None
-            if r'timestamp' in kw:
-                timestamp = kw.pop(r'timestamp')
+            if 'timestamp' in kw:
+                timestamp = kw.pop('timestamp')
             if timestamp is None:
                 self.timestamp = time.time()
             else:
@@ -137,45 +147,50 @@ class tarit(object):
             gzip.GzipFile.__init__(self, *args, **kw)
 
         def _write_gzip_header(self):
-            self.fileobj.write('\037\213')             # magic header
-            self.fileobj.write('\010')                 # compression method
+            self.fileobj.write(b'\037\213')  # magic header
+            self.fileobj.write(b'\010')  # compression method
             fname = self.name
-            if fname and fname.endswith('.gz'):
+            if fname and fname.endswith(b'.gz'):
                 fname = fname[:-3]
             flags = 0
             if fname:
-                flags = gzip.FNAME
+                flags = gzip.FNAME  # pytype: disable=module-attr
             self.fileobj.write(pycompat.bytechr(flags))
-            gzip.write32u(self.fileobj, int(self.timestamp))
-            self.fileobj.write('\002')
-            self.fileobj.write('\377')
+            gzip.write32u(  # pytype: disable=module-attr
+                self.fileobj, int(self.timestamp)
+            )
+            self.fileobj.write(b'\002')
+            self.fileobj.write(b'\377')
             if fname:
-                self.fileobj.write(fname + '\000')
+                self.fileobj.write(fname + b'\000')
 
-    def __init__(self, dest, mtime, kind=''):
+    def __init__(self, dest, mtime, kind=b''):
         self.mtime = mtime
         self.fileobj = None
 
-        def taropen(mode, name='', fileobj=None):
-            if kind == 'gz':
+        def taropen(mode, name=b'', fileobj=None):
+            if kind == b'gz':
                 mode = mode[0:1]
                 if not fileobj:
-                    fileobj = open(name, mode + 'b')
-                gzfileobj = self.GzipFileWithTime(name,
-                                                  pycompat.sysstr(mode + 'b'),
-                                                  zlib.Z_BEST_COMPRESSION,
-                                                  fileobj, timestamp=mtime)
+                    fileobj = open(name, mode + b'b')
+                gzfileobj = self.GzipFileWithTime(
+                    name,
+                    pycompat.sysstr(mode + b'b'),
+                    zlib.Z_BEST_COMPRESSION,
+                    fileobj,
+                    timestamp=mtime,
+                )
                 self.fileobj = gzfileobj
-                return tarfile.TarFile.taropen(
-                    name, pycompat.sysstr(mode), gzfileobj)
+                return tarfile.TarFile.taropen(  # pytype: disable=attribute-error
+                    name, pycompat.sysstr(mode), gzfileobj
+                )
             else:
-                return tarfile.open(
-                    name, pycompat.sysstr(mode + kind), fileobj)
+                return tarfile.open(name, pycompat.sysstr(mode + kind), fileobj)
 
         if isinstance(dest, bytes):
-            self.z = taropen('w:', name=dest)
+            self.z = taropen(b'w:', name=dest)
         else:
-            self.z = taropen('w|', fileobj=dest)
+            self.z = taropen(b'w|', fileobj=dest)
 
     def addfile(self, name, mode, islink, data):
         name = pycompat.fsdecode(name)
@@ -198,6 +213,7 @@ class tarit(object):
         if self.fileobj:
             self.fileobj.close()
 
+
 class zipit(object):
     '''write archive to zip file or stream.  can write uncompressed,
     or compressed with deflate.'''
@@ -205,13 +221,13 @@ class zipit(object):
     def __init__(self, dest, mtime, compress=True):
         if isinstance(dest, bytes):
             dest = pycompat.fsdecode(dest)
-        self.z = zipfile.ZipFile(dest, r'w',
-                                 compress and zipfile.ZIP_DEFLATED or
-                                 zipfile.ZIP_STORED)
+        self.z = zipfile.ZipFile(
+            dest, 'w', compress and zipfile.ZIP_DEFLATED or zipfile.ZIP_STORED
+        )
 
         # Python's zipfile module emits deprecation warnings if we try
         # to store files with a date before 1980.
-        epoch = 315532800 # calendar.timegm((1980, 1, 1, 0, 0, 0, 1, 1, 0))
+        epoch = 315532800  # calendar.timegm((1980, 1, 1, 0, 0, 0, 1, 1, 0))
         if mtime < epoch:
             mtime = epoch
 
@@ -220,7 +236,7 @@ class zipit(object):
 
     def addfile(self, name, mode, islink, data):
         i = zipfile.ZipInfo(pycompat.fsdecode(name), self.date_time)
-        i.compress_type = self.z.compression
+        i.compress_type = self.z.compression  # pytype: disable=attribute-error
         # unzip will not honor unix file modes unless file creator is
         # set to unix (id 3).
         i.create_system = 3
@@ -232,16 +248,19 @@ class zipit(object):
         # add "extended-timestamp" extra block, because zip archives
         # without this will be extracted with unexpected timestamp,
         # if TZ is not configured as GMT
-        i.extra += struct.pack('<hhBl',
-                               0x5455,     # block type: "extended-timestamp"
-                               1 + 4,      # size of this block
-                               1,          # "modification time is present"
-                               int(self.mtime)) # last modification (UTC)
+        i.extra += struct.pack(
+            b'<hhBl',
+            0x5455,  # block type: "extended-timestamp"
+            1 + 4,  # size of this block
+            1,  # "modification time is present"
+            int(self.mtime),
+        )  # last modification (UTC)
         self.z.writestr(i, data)
 
     def done(self):
         self.z.close()
 
+
 class fileit(object):
     '''write archive as files in directory.'''
 
@@ -254,7 +273,7 @@ class fileit(object):
         if islink:
             self.opener.symlink(data, name)
             return
-        f = self.opener(name, "w", atomictemp=False)
+        f = self.opener(name, b"w", atomictemp=False)
         f.write(data)
         f.close()
         destfile = os.path.join(self.basedir, name)
@@ -265,17 +284,29 @@ class fileit(object):
     def done(self):
         pass
 
-archivers = {
-    'files': fileit,
-    'tar': tarit,
-    'tbz2': lambda name, mtime: tarit(name, mtime, 'bz2'),
-    'tgz': lambda name, mtime: tarit(name, mtime, 'gz'),
-    'uzip': lambda name, mtime: zipit(name, mtime, False),
-    'zip': zipit,
-    }
 
-def archive(repo, dest, node, kind, decode=True, match=None,
-            prefix='', mtime=None, subrepos=False):
+archivers = {
+    b'files': fileit,
+    b'tar': tarit,
+    b'tbz2': lambda name, mtime: tarit(name, mtime, b'bz2'),
+    b'tgz': lambda name, mtime: tarit(name, mtime, b'gz'),
+    b'txz': lambda name, mtime: tarit(name, mtime, b'xz'),
+    b'uzip': lambda name, mtime: zipit(name, mtime, False),
+    b'zip': zipit,
+}
+
+
+def archive(
+    repo,
+    dest,
+    node,
+    kind,
+    decode=True,
+    match=None,
+    prefix=b'',
+    mtime=None,
+    subrepos=False,
+):
     '''create archive of repo as it was at node.
 
     dest can be name of directory, name of archive file, or file
@@ -295,9 +326,12 @@ def archive(repo, dest, node, kind, decode=True, match=None,
     subrepos tells whether to include subrepos.
     '''
 
-    if kind == 'files':
+    if kind == b'txz' and not pycompat.ispy3:
+        raise error.Abort(_(b'xz compression is only available in Python 3'))
+
+    if kind == b'files':
         if prefix:
-            raise error.Abort(_('cannot give prefix when archiving to files'))
+            raise error.Abort(_(b'cannot give prefix when archiving to files'))
     else:
         prefix = tidyprefix(dest, kind, prefix)
 
@@ -308,7 +342,7 @@ def archive(repo, dest, node, kind, decode=True, match=None,
         archiver.addfile(prefix + name, mode, islink, data)
 
     if kind not in archivers:
-        raise error.Abort(_("unknown archive type '%s'") % kind)
+        raise error.Abort(_(b"unknown archive type '%s'") % kind)
 
     ctx = repo[node]
     archiver = archivers[kind](dest, mtime or ctx.date()[0])
@@ -316,8 +350,8 @@ def archive(repo, dest, node, kind, decode=True, match=None,
     if not match:
         match = scmutil.matchall(repo)
 
-    if repo.ui.configbool("ui", "archivemeta"):
-        name = '.hg_archival.txt'
+    if repo.ui.configbool(b"ui", b"archivemeta"):
+        name = b'.hg_archival.txt'
         if match(name):
             write(name, 0o644, False, lambda: buildmetadata(ctx))
 
@@ -325,14 +359,16 @@ def archive(repo, dest, node, kind, decode=True, match=None,
     total = len(files)
     if total:
         files.sort()
-        scmutil.prefetchfiles(repo, [ctx.rev()],
-                              scmutil.matchfiles(repo, files))
-        progress = repo.ui.makeprogress(_('archiving'), unit=_('files'),
-                                        total=total)
+        scmutil.prefetchfiles(
+            repo, [ctx.rev()], scmutil.matchfiles(repo, files)
+        )
+        progress = repo.ui.makeprogress(
+            _(b'archiving'), unit=_(b'files'), total=total
+        )
         progress.update(0)
         for f in files:
             ff = ctx.flags(f)
-            write(f, 'x' in ff and 0o755 or 0o644, 'l' in ff, ctx[f].data)
+            write(f, b'x' in ff and 0o755 or 0o644, b'l' in ff, ctx[f].data)
             progress.increment(item=f)
         progress.complete()
 
@@ -340,11 +376,11 @@ def archive(repo, dest, node, kind, decode=True, match=None,
         for subpath in sorted(ctx.substate):
             sub = ctx.workingsub(subpath)
             submatch = matchmod.subdirmatcher(subpath, match)
-            subprefix = prefix + subpath + '/'
+            subprefix = prefix + subpath + b'/'
             total += sub.archive(archiver, subprefix, submatch, decode)
 
     if total == 0:
-        raise error.Abort(_('no files match the archive pattern'))
+        raise error.Abort(_(b'no files match the archive pattern'))
 
     archiver.done()
     return total
