@@ -15,11 +15,10 @@ from . import (
     encoding,
     node as nodemod,
     phases,
+    pycompat,
     util,
 )
-from .utils import (
-    dateutil,
-)
+from .utils import dateutil
 
 ### obsolescence marker flag
 
@@ -54,6 +53,7 @@ from .utils import (
 bumpedfix = 1
 usingsha256 = 2
 
+
 class marker(object):
     """Wrap obsolete marker raw data"""
 
@@ -95,6 +95,7 @@ class marker(object):
         """The flags field of the marker"""
         return self._data[2]
 
+
 def getmarkers(repo, nodes=None, exclusive=False):
     """returns markers known in a repository
 
@@ -110,6 +111,12 @@ def getmarkers(repo, nodes=None, exclusive=False):
     for markerdata in rawmarkers:
         yield marker(repo, markerdata)
 
+
+def sortedmarkers(markers):
+    # last item of marker tuple ('parents') may be None or a tuple
+    return sorted(markers, key=lambda m: m[:-1] + (m[-1] or (),))
+
+
 def closestpredecessors(repo, nodeid):
     """yield the list of next predecessors pointing on visible changectx nodes
 
@@ -138,6 +145,7 @@ def closestpredecessors(repo, nodeid):
             else:
                 stack.append(precnodeid)
 
+
 def allpredecessors(obsstore, nodes, ignoreflags=0):
     """Yield node for every precursors of <nodes>.
 
@@ -148,10 +156,11 @@ def allpredecessors(obsstore, nodes, ignoreflags=0):
 
     remaining = set(nodes)
     seen = set(remaining)
+    prec = obsstore.predecessors.get
     while remaining:
         current = remaining.pop()
         yield current
-        for mark in obsstore.predecessors.get(current, ()):
+        for mark in prec(current, ()):
             # ignore marker flagged with specified flag
             if mark[2] & ignoreflags:
                 continue
@@ -160,6 +169,7 @@ def allpredecessors(obsstore, nodes, ignoreflags=0):
                 seen.add(suc)
                 remaining.add(suc)
 
+
 def allsuccessors(obsstore, nodes, ignoreflags=0):
     """Yield node for every successor of <nodes>.
 
@@ -181,10 +191,12 @@ def allsuccessors(obsstore, nodes, ignoreflags=0):
                     seen.add(suc)
                     remaining.add(suc)
 
+
 def _filterprunes(markers):
     """return a set with no prune markers"""
     return set(m for m in markers if m[1])
 
+
 def exclusivemarkers(repo, nodes):
     """set of markers relevant to "nodes" but no other locally-known nodes
 
@@ -242,7 +254,7 @@ def exclusivemarkers(repo, nodes):
     unfi = repo.unfiltered()
 
     # shortcut to various useful item
-    nm = unfi.changelog.nodemap
+    has_node = unfi.changelog.index.has_node
     precursorsmarkers = unfi.obsstore.predecessors
     successormarkers = unfi.obsstore.successors
     childrenmarkers = unfi.obsstore.children
@@ -290,7 +302,7 @@ def exclusivemarkers(repo, nodes):
                 continue
 
             # is this a locally known node ?
-            known = prec in nm
+            known = has_node(prec)
             # if locally-known and not in the <nodes> set the traversal
             # stop here.
             if known and prec not in nodes:
@@ -306,6 +318,7 @@ def exclusivemarkers(repo, nodes):
 
     return exclmarkers
 
+
 def foreground(repo, nodes):
     """return all nodes in the "foreground" of other node
 
@@ -316,11 +329,11 @@ def foreground(repo, nodes):
     Beware that possible obsolescence cycle may result if complex situation.
     """
     repo = repo.unfiltered()
-    foreground = set(repo.set('%ln::', nodes))
+    foreground = set(repo.set(b'%ln::', nodes))
     if repo.obsstore:
         # We only need this complicated logic if there is obsolescence
         # XXX will probably deserve an optimised revset.
-        nm = repo.changelog.nodemap
+        has_node = repo.changelog.index.has_node
         plen = -1
         # compute the whole set of successors or descendants
         while len(foreground) != plen:
@@ -328,10 +341,11 @@ def foreground(repo, nodes):
             succs = set(c.node() for c in foreground)
             mutable = [c.node() for c in foreground if c.mutable()]
             succs.update(allsuccessors(repo.obsstore, mutable))
-            known = (n for n in succs if n in nm)
-            foreground = set(repo.set('%ln::', known))
+            known = (n for n in succs if has_node(n))
+            foreground = set(repo.set(b'%ln::', known))
     return set(c.node() for c in foreground)
 
+
 # effectflag field
 #
 # Effect-flag is a 1-byte bit field used to store what changed between a
@@ -347,23 +361,24 @@ def foreground(repo, nodes):
 # `effect-flags` set to off by default.
 #
 
-EFFECTFLAGFIELD = "ef1"
+EFFECTFLAGFIELD = b"ef1"
 
-DESCCHANGED = 1 << 0 # action changed the description
-METACHANGED = 1 << 1 # action change the meta
-DIFFCHANGED = 1 << 3 # action change diff introduced by the changeset
-PARENTCHANGED = 1 << 2 # action change the parent
-USERCHANGED = 1 << 4 # the user changed
-DATECHANGED = 1 << 5 # the date changed
-BRANCHCHANGED = 1 << 6 # the branch changed
+DESCCHANGED = 1 << 0  # action changed the description
+METACHANGED = 1 << 1  # action change the meta
+DIFFCHANGED = 1 << 3  # action change diff introduced by the changeset
+PARENTCHANGED = 1 << 2  # action change the parent
+USERCHANGED = 1 << 4  # the user changed
+DATECHANGED = 1 << 5  # the date changed
+BRANCHCHANGED = 1 << 6  # the branch changed
 
 METABLACKLIST = [
-    re.compile('^branch$'),
-    re.compile('^.*-source$'),
-    re.compile('^.*_source$'),
-    re.compile('^source$'),
+    re.compile(b'^branch$'),
+    re.compile(b'^.*-source$'),
+    re.compile(b'^.*_source$'),
+    re.compile(b'^source$'),
 ]
 
+
 def metanotblacklisted(metaitem):
     """ Check that the key of a meta item (extrakey, extravalue) does not
     match at least one of the blacklist pattern
@@ -372,6 +387,7 @@ def metanotblacklisted(metaitem):
 
     return not any(pattern.match(metakey) for pattern in METABLACKLIST)
 
+
 def _prepare_hunk(hunk):
     """Drop all information but the username and patch"""
     cleanhunk = []
@@ -382,6 +398,7 @@ def _prepare_hunk(hunk):
             cleanhunk.append(line)
     return cleanhunk
 
+
 def _getdifflines(iterdiff):
     """return a cleaned up lines"""
     lines = next(iterdiff, None)
@@ -391,14 +408,20 @@ def _getdifflines(iterdiff):
 
     return _prepare_hunk(lines)
 
+
 def _cmpdiff(leftctx, rightctx):
     """return True if both ctx introduce the "same diff"
 
     This is a first and basic implementation, with many shortcoming.
     """
-    diffopts = diffutil.diffallopts(leftctx.repo().ui, {'git': True})
+    diffopts = diffutil.diffallopts(leftctx.repo().ui, {b'git': True})
+
     # Leftctx or right ctx might be filtered, so we need to use the contexts
     # with an unfiltered repository to safely compute the diff
+
+    # leftctx and rightctx can be from different repository views in case of
+    # hgsubversion, do don't try to access them from same repository
+    # rightctx.repo() and leftctx.repo() are not always the same
     leftunfi = leftctx._repo.unfiltered()[leftctx.rev()]
     leftdiff = leftunfi.diff(opts=diffopts)
     rightunfi = rightctx._repo.unfiltered()[rightctx.rev()]
@@ -413,6 +436,7 @@ def _cmpdiff(leftctx, rightctx):
             return False
     return True
 
+
 def geteffectflag(source, successors):
     """ From an obs-marker relation, compute what changed between the
     predecessor and the successor.
@@ -456,14 +480,15 @@ def geteffectflag(source, successors):
 
     return effects
 
+
 def getobsoleted(repo, tr):
     """return the set of pre-existing revisions obsoleted by a transaction"""
-    torev = repo.unfiltered().changelog.nodemap.get
+    torev = repo.unfiltered().changelog.index.get_rev
     phase = repo._phasecache.phase
     succsmarkers = repo.obsstore.successors.get
     public = phases.public
-    addedmarkers = tr.changes['obsmarkers']
-    origrepolen = tr.changes['origrepolen']
+    addedmarkers = tr.changes[b'obsmarkers']
+    origrepolen = tr.changes[b'origrepolen']
     seenrevs = set()
     obsoleted = set()
     for mark in addedmarkers:
@@ -478,6 +503,7 @@ def getobsoleted(repo, tr):
             obsoleted.add(rev)
     return obsoleted
 
+
 class _succs(list):
     """small class to represent a successors with some metadata about it"""
 
@@ -498,6 +524,7 @@ class _succs(list):
     def canmerge(self, other):
         return self._set.issubset(other._set)
 
+
 def successorssets(repo, initialnode, closest=False, cache=None):
     """Return set of all latest successors of initial nodes
 
@@ -605,9 +632,9 @@ def successorssets(repo, initialnode, closest=False, cache=None):
 
         # case 2 condition is a bit hairy because of closest,
         # we compute it on its own
-        case2condition =  ((current not in succmarkers)
-                           or (closest and current != initialnode
-                               and current in repo))
+        case2condition = (current not in succmarkers) or (
+            closest and current != initialnode and current in repo
+        )
 
         if current in cache:
             # case (1): We already know the successors sets
@@ -653,7 +680,7 @@ def successorssets(repo, initialnode, closest=False, cache=None):
             #   Having none means pruned node, multiple successors means split,
             #   single successors are standard replacement.
             #
-            for mark in sorted(succmarkers[current]):
+            for mark in sortedmarkers(succmarkers[current]):
                 for suc in mark[1]:
                     if suc not in cache:
                         if suc in stackedset:
@@ -690,7 +717,7 @@ def successorssets(repo, initialnode, closest=False, cache=None):
                 # duplicated entry and successors set that are strict subset of
                 # another one.
                 succssets = []
-                for mark in sorted(succmarkers[current]):
+                for mark in sortedmarkers(succmarkers[current]):
                     # successors sets contributed by this marker
                     base = _succs()
                     base.markers.add(mark)
@@ -708,13 +735,15 @@ def successorssets(repo, initialnode, closest=False, cache=None):
                                     if part not in newss:
                                         newss.append(part)
                                 productresult.append(newss)
-                        markss = productresult
+                        if productresult:
+                            markss = productresult
                     succssets.extend(markss)
                 # remove duplicated and subset
                 seen = []
                 final = []
-                candidates = sorted((s for s in succssets if s),
-                                    key=len, reverse=True)
+                candidates = sorted(
+                    (s for s in succssets if s), key=len, reverse=True
+                )
                 for cand in candidates:
                     for seensuccs in seen:
                         if cand.canmerge(seensuccs):
@@ -723,10 +752,11 @@ def successorssets(repo, initialnode, closest=False, cache=None):
                     else:
                         final.append(cand)
                         seen.append(cand)
-                final.reverse() # put small successors set first
+                final.reverse()  # put small successors set first
                 cache[current] = final
     return cache[initialnode]
 
+
 def successorsandmarkers(repo, ctx):
     """compute the raw data needed for computing obsfate
     Returns a list of dict, one dict per successors set
@@ -743,7 +773,7 @@ def successorsandmarkers(repo, ctx):
 
     # Try to recover pruned markers
     succsmap = repo.obsstore.successors
-    fullsuccessorsets = [] # successor set + markers
+    fullsuccessorsets = []  # successor set + markers
     for sset in ssets:
         if sset:
             fullsuccessorsets.append(sset)
@@ -770,10 +800,11 @@ def successorsandmarkers(repo, ctx):
 
     values = []
     for sset in fullsuccessorsets:
-        values.append({'successors': sset, 'markers': sset.markers})
+        values.append({b'successors': sset, b'markers': sset.markers})
 
     return values
 
+
 def _getobsfate(successorssets):
     """ Compute a changeset obsolescence fate based on its successorssets.
     Successors can be the tipmost ones or the immediate ones. This function
@@ -788,53 +819,62 @@ def _getobsfate(successorssets):
 
     if len(successorssets) == 0:
         # The commit has been pruned
-        return 'pruned'
+        return b'pruned'
     elif len(successorssets) > 1:
-        return 'diverged'
+        return b'diverged'
     else:
         # No divergence, only one set of successors
         successors = successorssets[0]
 
         if len(successors) == 1:
-            return 'superseded'
+            return b'superseded'
         else:
-            return 'superseded_split'
+            return b'superseded_split'
+
 
 def obsfateverb(successorset, markers):
     """ Return the verb summarizing the successorset and potentially using
     information from the markers
     """
     if not successorset:
-        verb = 'pruned'
+        verb = b'pruned'
     elif len(successorset) == 1:
-        verb = 'rewritten'
+        verb = b'rewritten'
     else:
-        verb = 'split'
+        verb = b'split'
     return verb
 
+
 def markersdates(markers):
     """returns the list of dates for a list of markers
     """
     return [m[4] for m in markers]
 
+
 def markersusers(markers):
     """ Returns a sorted list of markers users without duplicates
     """
     markersmeta = [dict(m[3]) for m in markers]
-    users = set(encoding.tolocal(meta['user']) for meta in markersmeta
-                if meta.get('user'))
+    users = set(
+        encoding.tolocal(meta[b'user'])
+        for meta in markersmeta
+        if meta.get(b'user')
+    )
 
     return sorted(users)
 
+
 def markersoperations(markers):
     """ Returns a sorted list of markers operations without duplicates
     """
     markersmeta = [dict(m[3]) for m in markers]
-    operations = set(meta.get('operation') for meta in markersmeta
-                     if meta.get('operation'))
+    operations = set(
+        meta.get(b'operation') for meta in markersmeta if meta.get(b'operation')
+    )
 
     return sorted(operations)
 
+
 def obsfateprinter(ui, repo, successors, markers, formatctx):
     """ Build a obsfate string for a single successorset using all obsfate
     related function defined in obsutil
@@ -851,12 +891,12 @@ def obsfateprinter(ui, repo, successors, markers, formatctx):
     # Operations
     operations = markersoperations(markers)
     if operations:
-        line.append(" using %s" % ", ".join(operations))
+        line.append(b" using %s" % b", ".join(operations))
 
     # Successors
     if successors:
         fmtsuccessors = [formatctx(repo[succ]) for succ in successors]
-        line.append(" as %s" % ", ".join(fmtsuccessors))
+        line.append(b" as %s" % b", ".join(fmtsuccessors))
 
     # Users
     users = markersusers(markers)
@@ -868,7 +908,7 @@ def obsfateprinter(ui, repo, successors, markers, formatctx):
             users = None
 
     if (verbose or normal) and users:
-        line.append(" by %s" % ", ".join(users))
+        line.append(b" by %s" % b", ".join(users))
 
     # Date
     dates = markersdates(markers)
@@ -878,25 +918,27 @@ def obsfateprinter(ui, repo, successors, markers, formatctx):
         max_date = max(dates)
 
         if min_date == max_date:
-            fmtmin_date = dateutil.datestr(min_date, '%Y-%m-%d %H:%M %1%2')
-            line.append(" (at %s)" % fmtmin_date)
+            fmtmin_date = dateutil.datestr(min_date, b'%Y-%m-%d %H:%M %1%2')
+            line.append(b" (at %s)" % fmtmin_date)
         else:
-            fmtmin_date = dateutil.datestr(min_date, '%Y-%m-%d %H:%M %1%2')
-            fmtmax_date = dateutil.datestr(max_date, '%Y-%m-%d %H:%M %1%2')
-            line.append(" (between %s and %s)" % (fmtmin_date, fmtmax_date))
+            fmtmin_date = dateutil.datestr(min_date, b'%Y-%m-%d %H:%M %1%2')
+            fmtmax_date = dateutil.datestr(max_date, b'%Y-%m-%d %H:%M %1%2')
+            line.append(b" (between %s and %s)" % (fmtmin_date, fmtmax_date))
 
-    return "".join(line)
+    return b"".join(line)
 
 
 filteredmsgtable = {
-    "pruned": _("hidden revision '%s' is pruned"),
-    "diverged": _("hidden revision '%s' has diverged"),
-    "superseded": _("hidden revision '%s' was rewritten as: %s"),
-    "superseded_split": _("hidden revision '%s' was split as: %s"),
-    "superseded_split_several": _("hidden revision '%s' was split as: %s and "
-                                  "%d more"),
+    b"pruned": _(b"hidden revision '%s' is pruned"),
+    b"diverged": _(b"hidden revision '%s' has diverged"),
+    b"superseded": _(b"hidden revision '%s' was rewritten as: %s"),
+    b"superseded_split": _(b"hidden revision '%s' was split as: %s"),
+    b"superseded_split_several": _(
+        b"hidden revision '%s' was split as: %s and %d more"
+    ),
 }
 
+
 def _getfilteredreason(repo, changeid, ctx):
     """return a human-friendly string on why a obsolete changeset is hidden
     """
@@ -904,28 +946,29 @@ def _getfilteredreason(repo, changeid, ctx):
     fate = _getobsfate(successors)
 
     # Be more precise in case the revision is superseded
-    if fate == 'pruned':
-        return filteredmsgtable['pruned'] % changeid
-    elif fate == 'diverged':
-        return filteredmsgtable['diverged'] % changeid
-    elif fate == 'superseded':
+    if fate == b'pruned':
+        return filteredmsgtable[b'pruned'] % changeid
+    elif fate == b'diverged':
+        return filteredmsgtable[b'diverged'] % changeid
+    elif fate == b'superseded':
         single_successor = nodemod.short(successors[0][0])
-        return filteredmsgtable['superseded'] % (changeid, single_successor)
-    elif fate == 'superseded_split':
+        return filteredmsgtable[b'superseded'] % (changeid, single_successor)
+    elif fate == b'superseded_split':
 
         succs = []
         for node_id in successors[0]:
             succs.append(nodemod.short(node_id))
 
         if len(succs) <= 2:
-            fmtsuccs = ', '.join(succs)
-            return filteredmsgtable['superseded_split'] % (changeid, fmtsuccs)
+            fmtsuccs = b', '.join(succs)
+            return filteredmsgtable[b'superseded_split'] % (changeid, fmtsuccs)
         else:
-            firstsuccessors = ', '.join(succs[:2])
+            firstsuccessors = b', '.join(succs[:2])
             remainingnumber = len(succs) - 2
 
             args = (changeid, firstsuccessors, remainingnumber)
-            return filteredmsgtable['superseded_split_several'] % args
+            return filteredmsgtable[b'superseded_split_several'] % args
+
 
 def divergentsets(repo, ctx):
     """Compute sets of commits divergent with a given one"""
@@ -944,8 +987,11 @@ def divergentsets(repo, ctx):
                 # we already know the latest base for this divergency
                 continue
             base[tuple(nsuccset)] = n
-    return [{'divergentnodes': divset, 'commonpredecessor': b}
-            for divset, b in base.iteritems()]
+    return [
+        {b'divergentnodes': divset, b'commonpredecessor': b}
+        for divset, b in pycompat.iteritems(base)
+    ]
+
 
 def whyunstable(repo, ctx):
     result = []
@@ -953,28 +999,42 @@ def whyunstable(repo, ctx):
         for parent in ctx.parents():
             kind = None
             if parent.orphan():
-                kind = 'orphan'
+                kind = b'orphan'
             elif parent.obsolete():
-                kind = 'obsolete'
+                kind = b'obsolete'
             if kind is not None:
-                result.append({'instability': 'orphan',
-                               'reason': '%s parent' % kind,
-                               'node': parent.hex()})
+                result.append(
+                    {
+                        b'instability': b'orphan',
+                        b'reason': b'%s parent' % kind,
+                        b'node': parent.hex(),
+                    }
+                )
     if ctx.phasedivergent():
-        predecessors = allpredecessors(repo.obsstore, [ctx.node()],
-                                       ignoreflags=bumpedfix)
-        immutable = [repo[p] for p in predecessors
-                     if p in repo and not repo[p].mutable()]
+        predecessors = allpredecessors(
+            repo.obsstore, [ctx.node()], ignoreflags=bumpedfix
+        )
+        immutable = [
+            repo[p] for p in predecessors if p in repo and not repo[p].mutable()
+        ]
         for predecessor in immutable:
-            result.append({'instability': 'phase-divergent',
-                           'reason': 'immutable predecessor',
-                           'node': predecessor.hex()})
+            result.append(
+                {
+                    b'instability': b'phase-divergent',
+                    b'reason': b'immutable predecessor',
+                    b'node': predecessor.hex(),
+                }
+            )
     if ctx.contentdivergent():
         dsets = divergentsets(repo, ctx)
         for dset in dsets:
-            divnodes = [repo[n] for n in dset['divergentnodes']]
-            result.append({'instability': 'content-divergent',
-                           'divergentnodes': divnodes,
-                           'reason': 'predecessor',
-                           'node': nodemod.hex(dset['commonpredecessor'])})
+            divnodes = [repo[n] for n in dset[b'divergentnodes']]
+            result.append(
+                {
+                    b'instability': b'content-divergent',
+                    b'divergentnodes': divnodes,
+                    b'reason': b'predecessor',
+                    b'node': nodemod.hex(dset[b'commonpredecessor']),
+                }
+            )
     return result
