@@ -24,13 +24,13 @@ from . import (
     mdiff,
     pycompat,
 )
-from .utils import (
-    stringutil,
-)
+from .utils import stringutil
+
 
 class CantReprocessAndShowBase(Exception):
     pass
 
+
 def intersect(ra, rb):
     """Given two ranges return the range where they intersect or None.
 
@@ -53,23 +53,27 @@ def intersect(ra, rb):
     else:
         return None
 
+
 def compare_range(a, astart, aend, b, bstart, bend):
     """Compare a[astart:aend] == b[bstart:bend], without slicing.
     """
     if (aend - astart) != (bend - bstart):
         return False
-    for ia, ib in zip(pycompat.xrange(astart, aend),
-                      pycompat.xrange(bstart, bend)):
+    for ia, ib in zip(
+        pycompat.xrange(astart, aend), pycompat.xrange(bstart, bend)
+    ):
         if a[ia] != b[ib]:
             return False
     else:
         return True
 
+
 class Merge3Text(object):
     """3-way merge of texts.
 
     Given strings BASE, OTHER, THIS, tries to produce a combined text
     incorporating the changes from both BASE->OTHER and BASE->THIS."""
+
     def __init__(self, basetext, atext, btext, base=None, a=None, b=None):
         self.basetext = basetext
         self.atext = atext
@@ -84,50 +88,52 @@ class Merge3Text(object):
         self.a = a
         self.b = b
 
-    def merge_lines(self,
-                    name_a=None,
-                    name_b=None,
-                    name_base=None,
-                    start_marker='<<<<<<<',
-                    mid_marker='=======',
-                    end_marker='>>>>>>>',
-                    base_marker=None,
-                    localorother=None,
-                    minimize=False):
+    def merge_lines(
+        self,
+        name_a=None,
+        name_b=None,
+        name_base=None,
+        start_marker=b'<<<<<<<',
+        mid_marker=b'=======',
+        end_marker=b'>>>>>>>',
+        base_marker=None,
+        localorother=None,
+        minimize=False,
+    ):
         """Return merge in cvs-like form.
         """
         self.conflicts = False
-        newline = '\n'
+        newline = b'\n'
         if len(self.a) > 0:
-            if self.a[0].endswith('\r\n'):
-                newline = '\r\n'
-            elif self.a[0].endswith('\r'):
-                newline = '\r'
+            if self.a[0].endswith(b'\r\n'):
+                newline = b'\r\n'
+            elif self.a[0].endswith(b'\r'):
+                newline = b'\r'
         if name_a and start_marker:
-            start_marker = start_marker + ' ' + name_a
+            start_marker = start_marker + b' ' + name_a
         if name_b and end_marker:
-            end_marker = end_marker + ' ' + name_b
+            end_marker = end_marker + b' ' + name_b
         if name_base and base_marker:
-            base_marker = base_marker + ' ' + name_base
+            base_marker = base_marker + b' ' + name_base
         merge_regions = self.merge_regions()
         if minimize:
             merge_regions = self.minimize(merge_regions)
         for t in merge_regions:
             what = t[0]
-            if what == 'unchanged':
+            if what == b'unchanged':
                 for i in range(t[1], t[2]):
                     yield self.base[i]
-            elif what == 'a' or what == 'same':
+            elif what == b'a' or what == b'same':
                 for i in range(t[1], t[2]):
                     yield self.a[i]
-            elif what == 'b':
+            elif what == b'b':
                 for i in range(t[1], t[2]):
                     yield self.b[i]
-            elif what == 'conflict':
-                if localorother == 'local':
+            elif what == b'conflict':
+                if localorother == b'local':
                     for i in range(t[3], t[4]):
                         yield self.a[i]
-                elif localorother == 'other':
+                elif localorother == b'other':
                     for i in range(t[5], t[6]):
                         yield self.b[i]
                 else:
@@ -169,17 +175,19 @@ class Merge3Text(object):
         """
         for t in self.merge_regions():
             what = t[0]
-            if what == 'unchanged':
-                yield what, self.base[t[1]:t[2]]
-            elif what == 'a' or what == 'same':
-                yield what, self.a[t[1]:t[2]]
-            elif what == 'b':
-                yield what, self.b[t[1]:t[2]]
-            elif what == 'conflict':
-                yield (what,
-                       self.base[t[1]:t[2]],
-                       self.a[t[3]:t[4]],
-                       self.b[t[5]:t[6]])
+            if what == b'unchanged':
+                yield what, self.base[t[1] : t[2]]
+            elif what == b'a' or what == b'same':
+                yield what, self.a[t[1] : t[2]]
+            elif what == b'b':
+                yield what, self.b[t[1] : t[2]]
+            elif what == b'conflict':
+                yield (
+                    what,
+                    self.base[t[1] : t[2]],
+                    self.a[t[3] : t[4]],
+                    self.b[t[5] : t[6]],
+                )
             else:
                 raise ValueError(what)
 
@@ -218,7 +226,7 @@ class Merge3Text(object):
 
         for region in self.find_sync_regions():
             zmatch, zend, amatch, aend, bmatch, bend = region
-            #print 'match base [%d:%d]' % (zmatch, zend)
+            # print 'match base [%d:%d]' % (zmatch, zend)
 
             matchlen = zend - zmatch
             assert matchlen >= 0
@@ -232,27 +240,28 @@ class Merge3Text(object):
             assert len_b >= 0
             assert len_base >= 0
 
-            #print 'unmatched a=%d, b=%d' % (len_a, len_b)
+            # print 'unmatched a=%d, b=%d' % (len_a, len_b)
 
             if len_a or len_b:
                 # try to avoid actually slicing the lists
-                equal_a = compare_range(self.a, ia, amatch,
-                                        self.base, iz, zmatch)
-                equal_b = compare_range(self.b, ib, bmatch,
-                                        self.base, iz, zmatch)
-                same = compare_range(self.a, ia, amatch,
-                                     self.b, ib, bmatch)
+                equal_a = compare_range(
+                    self.a, ia, amatch, self.base, iz, zmatch
+                )
+                equal_b = compare_range(
+                    self.b, ib, bmatch, self.base, iz, zmatch
+                )
+                same = compare_range(self.a, ia, amatch, self.b, ib, bmatch)
 
                 if same:
-                    yield 'same', ia, amatch
+                    yield b'same', ia, amatch
                 elif equal_a and not equal_b:
-                    yield 'b', ib, bmatch
+                    yield b'b', ib, bmatch
                 elif equal_b and not equal_a:
-                    yield 'a', ia, amatch
+                    yield b'a', ia, amatch
                 elif not equal_a and not equal_b:
-                    yield 'conflict', iz, zmatch, ia, amatch, ib, bmatch
+                    yield b'conflict', iz, zmatch, ia, amatch, ib, bmatch
                 else:
-                    raise AssertionError("can't handle a=b=base but unmatched")
+                    raise AssertionError(b"can't handle a=b=base but unmatched")
 
                 ia = amatch
                 ib = bmatch
@@ -261,13 +270,12 @@ class Merge3Text(object):
             # if the same part of the base was deleted on both sides
             # that's OK, we can just skip it.
 
-
             if matchlen > 0:
                 assert ia == amatch
                 assert ib == bmatch
                 assert iz == zmatch
 
-                yield 'unchanged', zmatch, zend
+                yield b'unchanged', zmatch, zend
                 iz = zend
                 ia = aend
                 ib = bend
@@ -280,36 +288,58 @@ class Merge3Text(object):
         region and are instead considered the same.
         """
         for region in merge_regions:
-            if region[0] != "conflict":
+            if region[0] != b"conflict":
                 yield region
                 continue
-            issue, z1, z2, a1, a2, b1, b2 = region
+            # pytype thinks this tuple contains only 3 things, but
+            # that's clearly not true because this code successfully
+            # executes. It might be wise to rework merge_regions to be
+            # some kind of attrs type.
+            (
+                issue,
+                z1,
+                z2,
+                a1,
+                a2,
+                b1,
+                b2,
+            ) = region  # pytype: disable=bad-unpacking
             alen = a2 - a1
             blen = b2 - b1
 
             # find matches at the front
             ii = 0
-            while ii < alen and ii < blen and \
-                  self.a[a1 + ii] == self.b[b1 + ii]:
+            while (
+                ii < alen and ii < blen and self.a[a1 + ii] == self.b[b1 + ii]
+            ):
                 ii += 1
             startmatches = ii
 
             # find matches at the end
             ii = 0
-            while ii < alen and ii < blen and \
-                  self.a[a2 - ii - 1] == self.b[b2 - ii - 1]:
+            while (
+                ii < alen
+                and ii < blen
+                and self.a[a2 - ii - 1] == self.b[b2 - ii - 1]
+            ):
                 ii += 1
             endmatches = ii
 
             if startmatches > 0:
-                yield 'same', a1, a1 + startmatches
+                yield b'same', a1, a1 + startmatches
 
-            yield ('conflict', z1, z2,
-                    a1 + startmatches, a2 - endmatches,
-                    b1 + startmatches, b2 - endmatches)
+            yield (
+                b'conflict',
+                z1,
+                z2,
+                a1 + startmatches,
+                a2 - endmatches,
+                b1 + startmatches,
+                b2 - endmatches,
+            )
 
             if endmatches > 0:
-                yield 'same', a2 - endmatches, a2
+                yield b'same', a2 - endmatches, a2
 
     def find_sync_regions(self):
         """Return a list of sync regions, where both descendants match the base.
@@ -350,14 +380,14 @@ class Merge3Text(object):
                 aend = asub + intlen
                 bend = bsub + intlen
 
-                assert self.base[intbase:intend] == self.a[asub:aend], \
-                       (self.base[intbase:intend], self.a[asub:aend])
+                assert self.base[intbase:intend] == self.a[asub:aend], (
+                    self.base[intbase:intend],
+                    self.a[asub:aend],
+                )
 
                 assert self.base[intbase:intend] == self.b[bsub:bend]
 
-                sl.append((intbase, intend,
-                           asub, aend,
-                           bsub, bend))
+                sl.append((intbase, intend, asub, aend, bsub, bend))
 
             # advance whichever one ends first in the base text
             if (abase + alen) < (bbase + blen):
@@ -397,25 +427,28 @@ class Merge3Text(object):
 
         return unc
 
+
 def _verifytext(text, path, ui, opts):
     """verifies that text is non-binary (unless opts[text] is passed,
     then we just warn)"""
     if stringutil.binary(text):
-        msg = _("%s looks like a binary file.") % path
-        if not opts.get('quiet'):
-            ui.warn(_('warning: %s\n') % msg)
-        if not opts.get('text'):
+        msg = _(b"%s looks like a binary file.") % path
+        if not opts.get(b'quiet'):
+            ui.warn(_(b'warning: %s\n') % msg)
+        if not opts.get(b'text'):
             raise error.Abort(msg)
     return text
 
+
 def _picklabels(defaults, overrides):
     if len(overrides) > 3:
-        raise error.Abort(_("can only specify three labels."))
+        raise error.Abort(_(b"can only specify three labels."))
     result = defaults[:]
     for i, override in enumerate(overrides):
         result[i] = override
     return result
 
+
 def simplemerge(ui, localctx, basectx, otherctx, **opts):
     """Performs the simplemerge algorithm.
 
@@ -433,12 +466,12 @@ def simplemerge(ui, localctx, basectx, otherctx, **opts):
         # repository usually sees) might be more useful.
         return _verifytext(ctx.decodeddata(), ctx.path(), ui, opts)
 
-    mode = opts.get('mode','merge')
+    mode = opts.get(b'mode', b'merge')
     name_a, name_b, name_base = None, None, None
-    if mode != 'union':
-        name_a, name_b, name_base = _picklabels([localctx.path(),
-                                                 otherctx.path(), None],
-                                                opts.get('label', []))
+    if mode != b'union':
+        name_a, name_b, name_base = _picklabels(
+            [localctx.path(), otherctx.path(), None], opts.get(b'label', [])
+        )
 
     try:
         localtext = readctx(localctx)
@@ -449,28 +482,29 @@ def simplemerge(ui, localctx, basectx, otherctx, **opts):
 
     m3 = Merge3Text(basetext, localtext, othertext)
     extrakwargs = {
-            "localorother": opts.get("localorother", None),
-            'minimize': True,
-        }
-    if mode == 'union':
-        extrakwargs['start_marker'] = None
-        extrakwargs['mid_marker'] = None
-        extrakwargs['end_marker'] = None
+        b"localorother": opts.get(b"localorother", None),
+        b'minimize': True,
+    }
+    if mode == b'union':
+        extrakwargs[b'start_marker'] = None
+        extrakwargs[b'mid_marker'] = None
+        extrakwargs[b'end_marker'] = None
     elif name_base is not None:
-        extrakwargs['base_marker'] = '|||||||'
-        extrakwargs['name_base'] = name_base
-        extrakwargs['minimize'] = False
+        extrakwargs[b'base_marker'] = b'|||||||'
+        extrakwargs[b'name_base'] = name_base
+        extrakwargs[b'minimize'] = False
 
-    mergedtext = ""
-    for line in m3.merge_lines(name_a=name_a, name_b=name_b,
-                               **pycompat.strkwargs(extrakwargs)):
-        if opts.get('print'):
+    mergedtext = b""
+    for line in m3.merge_lines(
+        name_a=name_a, name_b=name_b, **pycompat.strkwargs(extrakwargs)
+    ):
+        if opts.get(b'print'):
             ui.fout.write(line)
         else:
             mergedtext += line
 
-    if not opts.get('print'):
+    if not opts.get(b'print'):
         localctx.write(mergedtext, localctx.flags())
 
-    if m3.conflicts and not mode == 'union':
+    if m3.conflicts and not mode == b'union':
         return 1
