@@ -12,24 +12,26 @@
 
 from __future__ import absolute_import
 
-import array
 import errno
-import time
+import io
 
-from .i18n import _
 from .node import (
     bin,
     hex,
     nullid,
+    nullrev,
     short,
 )
+from .i18n import _
 from . import (
     encoding,
     error,
+    match as matchmod,
+    pycompat,
+    scmutil,
     util,
 )
-
-array = array.array
+from .utils import stringutil
 
 # Tags computation can be expensive and caches exist to make it fast in
 # the common case.
@@ -80,58 +82,168 @@ array = array.array
 # The most recent changeset (in terms of revlog ordering for the head
 # setting it) for each tag is last.
 
-def findglobaltags(ui, repo, alltags, tagtypes):
-    '''Find global tags in a repo.
 
-    "alltags" maps tag name to (node, hist) 2-tuples.
+def fnoderevs(ui, repo, revs):
+    """return the list of '.hgtags' fnodes used in a set revisions
 
-    "tagtypes" maps tag name to tag type. Global tags always have the
-    "global" tag type.
+    This is returned as list of unique fnodes. We use a list instead of a set
+    because order matters when it comes to tags."""
+    unfi = repo.unfiltered()
+    tonode = unfi.changelog.node
+    nodes = [tonode(r) for r in revs]
+    fnodes = _getfnodes(ui, repo, nodes)
+    fnodes = _filterfnodes(fnodes, nodes)
+    return fnodes
 
-    The "alltags" and "tagtypes" dicts are updated in place. Empty dicts
-    should be passed in.
+
+def _nulltonone(value):
+    """convert nullid to None
+
+    For tag value, nullid means "deleted". This small utility function helps
+    translating that to None."""
+    if value == nullid:
+        return None
+    return value
+
+
+def difftags(ui, repo, oldfnodes, newfnodes):
+    """list differences between tags expressed in two set of file-nodes
+
+    The list contains entries in the form: (tagname, oldvalue, new value).
+    None is used to expressed missing value:
+        ('foo', None, 'abcd') is a new tag,
+        ('bar', 'ef01', None) is a deletion,
+        ('baz', 'abcd', 'ef01') is a tag movement.
+    """
+    if oldfnodes == newfnodes:
+        return []
+    oldtags = _tagsfromfnodes(ui, repo, oldfnodes)
+    newtags = _tagsfromfnodes(ui, repo, newfnodes)
+
+    # list of (tag, old, new): None means missing
+    entries = []
+    for tag, (new, __) in newtags.items():
+        new = _nulltonone(new)
+        old, __ = oldtags.pop(tag, (None, None))
+        old = _nulltonone(old)
+        if old != new:
+            entries.append((tag, old, new))
+    # handle deleted tags
+    for tag, (old, __) in oldtags.items():
+        old = _nulltonone(old)
+        if old is not None:
+            entries.append((tag, old, None))
+    entries.sort()
+    return entries
+
+
+def writediff(fp, difflist):
+    """write tags diff information to a file.
+
+    Data are stored with a line based format:
+
+        <action> <hex-node> <tag-name>\n
+
+    Action are defined as follow:
+       -R tag is removed,
+       +A tag is added,
+       -M tag is moved (old value),
+       +M tag is moved (new value),
+
+    Example:
+
+         +A 875517b4806a848f942811a315a5bce30804ae85 t5
+
+    See documentation of difftags output for details about the input.
+    """
+    add = b'+A %s %s\n'
+    remove = b'-R %s %s\n'
+    updateold = b'-M %s %s\n'
+    updatenew = b'+M %s %s\n'
+    for tag, old, new in difflist:
+        # translate to hex
+        if old is not None:
+            old = hex(old)
+        if new is not None:
+            new = hex(new)
+        # write to file
+        if old is None:
+            fp.write(add % (new, tag))
+        elif new is None:
+            fp.write(remove % (old, tag))
+        else:
+            fp.write(updateold % (old, tag))
+            fp.write(updatenew % (new, tag))
+
+
+def findglobaltags(ui, repo):
+    '''Find global tags in a repo: return a tagsmap
+
+    tagsmap: tag name to (node, hist) 2-tuples.
 
     The tags cache is read and updated as a side-effect of calling.
     '''
-    # This is so we can be lazy and assume alltags contains only global
-    # tags when we pass it to _writetagcache().
-    assert len(alltags) == len(tagtypes) == 0, \
-           "findglobaltags() should be called first"
-
     (heads, tagfnode, valid, cachetags, shouldwrite) = _readtagcache(ui, repo)
     if cachetags is not None:
         assert not shouldwrite
         # XXX is this really 100% correct?  are there oddball special
         # cases where a global tag should outrank a local tag but won't,
         # because cachetags does not contain rank info?
-        _updatetags(cachetags, 'global', alltags, tagtypes)
-        return
+        alltags = {}
+        _updatetags(cachetags, alltags)
+        return alltags
 
-    seen = set()  # set of fnode
-    fctx = None
     for head in reversed(heads):  # oldest to newest
-        assert head in repo.changelog.nodemap, \
-               "tag cache returned bogus head %s" % short(head)
-
-        fnode = tagfnode.get(head)
-        if fnode and fnode not in seen:
-            seen.add(fnode)
-            if not fctx:
-                fctx = repo.filectx('.hgtags', fileid=fnode)
-            else:
-                fctx = fctx.filectx(fnode)
-
-            filetags = _readtags(ui, repo, fctx.data().splitlines(), fctx)
-            _updatetags(filetags, 'global', alltags, tagtypes)
+        assert repo.changelog.index.has_node(
+            head
+        ), b"tag cache returned bogus head %s" % short(head)
+    fnodes = _filterfnodes(tagfnode, reversed(heads))
+    alltags = _tagsfromfnodes(ui, repo, fnodes)
 
     # and update the cache (if necessary)
     if shouldwrite:
         _writetagcache(ui, repo, valid, alltags)
+    return alltags
+
+
+def _filterfnodes(tagfnode, nodes):
+    """return a list of unique fnodes
+
+    The order of this list matches the order of "nodes". Preserving this order
+    is important as reading tags in different order provides different
+    results."""
+    seen = set()  # set of fnode
+    fnodes = []
+    for no in nodes:  # oldest to newest
+        fnode = tagfnode.get(no)
+        if fnode and fnode not in seen:
+            seen.add(fnode)
+            fnodes.append(fnode)
+    return fnodes
+
+
+def _tagsfromfnodes(ui, repo, fnodes):
+    """return a tagsmap from a list of file-node
+
+    tagsmap: tag name to (node, hist) 2-tuples.
+
+    The order of the list matters."""
+    alltags = {}
+    fctx = None
+    for fnode in fnodes:
+        if fctx is None:
+            fctx = repo.filectx(b'.hgtags', fileid=fnode)
+        else:
+            fctx = fctx.filectx(fnode)
+        filetags = _readtags(ui, repo, fctx.data().splitlines(), fctx)
+        _updatetags(filetags, alltags)
+    return alltags
+
 
 def readlocaltags(ui, repo, alltags, tagtypes):
     '''Read local tags in repo. Update alltags and tagtypes.'''
     try:
-        data = repo.vfs.read("localtags")
+        data = repo.vfs.read(b"localtags")
     except IOError as inst:
         if inst.errno != errno.ENOENT:
             raise
@@ -140,18 +252,19 @@ def readlocaltags(ui, repo, alltags, tagtypes):
     # localtags is in the local encoding; re-encode to UTF-8 on
     # input for consistency with the rest of this module.
     filetags = _readtags(
-        ui, repo, data.splitlines(), "localtags",
-        recode=encoding.fromlocal)
+        ui, repo, data.splitlines(), b"localtags", recode=encoding.fromlocal
+    )
 
     # remove tags pointing to invalid nodes
     cl = repo.changelog
-    for t in filetags.keys():
+    for t in list(filetags):
         try:
             cl.rev(filetags[t][0])
         except (LookupError, ValueError):
             del filetags[t]
 
-    _updatetags(filetags, "local", alltags, tagtypes)
+    _updatetags(filetags, alltags, b'local', tagtypes)
+
 
 def _readtaghist(ui, repo, lines, fn, recode=None, calcnodelines=False):
     '''Read tag definitions from a file (or any source of lines).
@@ -176,17 +289,17 @@ def _readtaghist(ui, repo, lines, fn, recode=None, calcnodelines=False):
     hextaglines = util.sortdict()
     count = 0
 
-    def warn(msg):
-        ui.warn(_("%s, line %s: %s\n") % (fn, count, msg))
+    def dbg(msg):
+        ui.debug(b"%s, line %d: %s\n" % (fn, count, msg))
 
     for nline, line in enumerate(lines):
         count += 1
         if not line:
             continue
         try:
-            (nodehex, name) = line.split(" ", 1)
+            (nodehex, name) = line.split(b" ", 1)
         except ValueError:
-            warn(_("cannot parse entry"))
+            dbg(b"cannot parse entry")
             continue
         name = name.strip()
         if recode:
@@ -194,7 +307,7 @@ def _readtaghist(ui, repo, lines, fn, recode=None, calcnodelines=False):
         try:
             nodebin = bin(nodehex)
         except TypeError:
-            warn(_("node '%s' is not well formed") % nodehex)
+            dbg(b"node '%s' is not well formed" % nodehex)
             continue
 
         # update filetags
@@ -210,6 +323,7 @@ def _readtaghist(ui, repo, lines, fn, recode=None, calcnodelines=False):
         bintaghist[name].append(nodebin)
     return bintaghist, hextaglines
 
+
 def _readtags(ui, repo, lines, fn, recode=None, calcnodelines=False):
     '''Read tag definitions from a file (or any source of lines).
 
@@ -219,8 +333,9 @@ def _readtags(ui, repo, lines, fn, recode=None, calcnodelines=False):
     is the list of node ids previously associated with it (in file order).
     All node ids are binary, not hex.
     '''
-    filetags, nodelines = _readtaghist(ui, repo, lines, fn, recode=recode,
-                                       calcnodelines=calcnodelines)
+    filetags, nodelines = _readtaghist(
+        ui, repo, lines, fn, recode=recode, calcnodelines=calcnodelines
+    )
     # util.sortdict().__setitem__ is much slower at replacing then inserting
     # new entries. The difference can matter if there are thousands of tags.
     # Create a new sortdict to avoid the performance penalty.
@@ -229,15 +344,23 @@ def _readtags(ui, repo, lines, fn, recode=None, calcnodelines=False):
         newtags[tag] = (taghist[-1], taghist[:-1])
     return newtags
 
-def _updatetags(filetags, tagtype, alltags, tagtypes):
-    '''Incorporate the tag info read from one file into the two
-    dictionaries, alltags and tagtypes, that contain all tag
-    info (global across all heads plus local).'''
 
-    for name, nodehist in filetags.iteritems():
+def _updatetags(filetags, alltags, tagtype=None, tagtypes=None):
+    """Incorporate the tag info read from one file into dictionnaries
+
+    The first one, 'alltags', is a "tagmaps" (see 'findglobaltags' for details).
+
+    The second one, 'tagtypes', is optional and will be updated to track the
+    "tagtype" of entries in the tagmaps. When set, the 'tagtype' argument also
+    needs to be set."""
+    if tagtype is None:
+        assert tagtypes is None
+
+    for name, nodehist in pycompat.iteritems(filetags):
         if name not in alltags:
             alltags[name] = nodehist
-            tagtypes[name] = tagtype
+            if tagtype is not None:
+                tagtypes[name] = tagtype
             continue
 
         # we prefer alltags[name] if:
@@ -246,21 +369,26 @@ def _updatetags(filetags, tagtype, alltags, tagtypes):
         # otherwise we win because we're tip-most
         anode, ahist = nodehist
         bnode, bhist = alltags[name]
-        if (bnode != anode and anode in bhist and
-            (bnode not in ahist or len(bhist) > len(ahist))):
+        if (
+            bnode != anode
+            and anode in bhist
+            and (bnode not in ahist or len(bhist) > len(ahist))
+        ):
             anode = bnode
-        else:
+        elif tagtype is not None:
             tagtypes[name] = tagtype
         ahist.extend([n for n in bhist if n not in ahist])
         alltags[name] = anode, ahist
 
+
 def _filename(repo):
     """name of a tagcache file for a given repo or repoview"""
-    filename = 'cache/tags2'
+    filename = b'tags2'
     if repo.filtername:
-        filename = '%s-%s' % (filename, repo.filtername)
+        filename = b'%s-%s' % (filename, repo.filtername)
     return filename
 
+
 def _readtagcache(ui, repo):
     '''Read the tag cache.
 
@@ -279,10 +407,8 @@ def _readtagcache(ui, repo):
     If the cache is not up to date, the caller is responsible for reading tag
     info from each returned head. (See findglobaltags().)
     '''
-    from . import scmutil  # avoid cycle
-
     try:
-        cachefile = repo.vfs(_filename(repo), 'r')
+        cachefile = repo.cachevfs(_filename(repo), b'r')
         # force reading the file for static-http
         cachelines = iter(cachefile)
     except IOError:
@@ -293,7 +419,7 @@ def _readtagcache(ui, repo):
     cachehash = None
     if cachefile:
         try:
-            validline = cachelines.next()
+            validline = next(cachelines)
             validline = validline.split()
             cacherev = int(validline[0])
             cachenode = bin(validline[1])
@@ -310,14 +436,16 @@ def _readtagcache(ui, repo):
     # (Unchanged tip trivially means no changesets have been added.
     # But, thanks to localrepository.destroyed(), it also means none
     # have been destroyed by strip or rollback.)
-    if (cacherev == tiprev
-            and cachenode == tipnode
-            and cachehash == scmutil.filteredhash(repo, tiprev)):
+    if (
+        cacherev == tiprev
+        and cachenode == tipnode
+        and cachehash == scmutil.filteredhash(repo, tiprev)
+    ):
         tags = _readtags(ui, repo, cachelines, cachefile.name)
         cachefile.close()
         return (None, None, None, tags, False)
     if cachefile:
-        cachefile.close()               # ignore rest of file
+        cachefile.close()  # ignore rest of file
 
     valid = (tiprev, tipnode, scmutil.filteredhash(repo, tiprev))
 
@@ -340,68 +468,210 @@ def _readtagcache(ui, repo):
 
     # N.B. in case 4 (nodes destroyed), "new head" really means "newly
     # exposed".
-    if not len(repo.file('.hgtags')):
+    if not len(repo.file(b'.hgtags')):
         # No tags have ever been committed, so we can avoid a
         # potentially expensive search.
         return ([], {}, valid, None, True)
 
-    starttime = time.time()
-
     # Now we have to lookup the .hgtags filenode for every new head.
     # This is the most expensive part of finding tags, so performance
     # depends primarily on the size of newheads.  Worst case: no cache
     # file, so newheads == repoheads.
-    fnodescache = hgtagsfnodescache(repo.unfiltered())
-    cachefnode = {}
-    for head in reversed(repoheads):
-        fnode = fnodescache.getfnode(head)
-        if fnode != nullid:
-            cachefnode[head] = fnode
-
-    fnodescache.write()
-
-    duration = time.time() - starttime
-    ui.log('tagscache',
-           '%d/%d cache hits/lookups in %0.4f '
-           'seconds\n',
-           fnodescache.hitcount, fnodescache.lookupcount, duration)
+    # Reversed order helps the cache ('repoheads' is in descending order)
+    cachefnode = _getfnodes(ui, repo, reversed(repoheads))
 
     # Caller has to iterate over all heads, but can use the filenodes in
     # cachefnode to get to each .hgtags revision quickly.
     return (repoheads, cachefnode, valid, None, True)
 
+
+def _getfnodes(ui, repo, nodes):
+    """return .hgtags fnodes for a list of changeset nodes
+
+    Return value is a {node: fnode} mapping. There will be no entry for nodes
+    without a '.hgtags' file.
+    """
+    starttime = util.timer()
+    fnodescache = hgtagsfnodescache(repo.unfiltered())
+    cachefnode = {}
+    for node in nodes:
+        fnode = fnodescache.getfnode(node)
+        if fnode != nullid:
+            cachefnode[node] = fnode
+
+    fnodescache.write()
+
+    duration = util.timer() - starttime
+    ui.log(
+        b'tagscache',
+        b'%d/%d cache hits/lookups in %0.4f seconds\n',
+        fnodescache.hitcount,
+        fnodescache.lookupcount,
+        duration,
+    )
+    return cachefnode
+
+
 def _writetagcache(ui, repo, valid, cachetags):
     filename = _filename(repo)
     try:
-        cachefile = repo.vfs(filename, 'w', atomictemp=True)
+        cachefile = repo.cachevfs(filename, b'w', atomictemp=True)
     except (OSError, IOError):
         return
 
-    ui.log('tagscache', 'writing .hg/%s with %d tags\n',
-           filename, len(cachetags))
+    ui.log(
+        b'tagscache',
+        b'writing .hg/cache/%s with %d tags\n',
+        filename,
+        len(cachetags),
+    )
 
     if valid[2]:
-        cachefile.write('%d %s %s\n' % (valid[0], hex(valid[1]), hex(valid[2])))
+        cachefile.write(
+            b'%d %s %s\n' % (valid[0], hex(valid[1]), hex(valid[2]))
+        )
     else:
-        cachefile.write('%d %s\n' % (valid[0], hex(valid[1])))
+        cachefile.write(b'%d %s\n' % (valid[0], hex(valid[1])))
 
     # Tag names in the cache are in UTF-8 -- which is the whole reason
     # we keep them in UTF-8 throughout this module.  If we converted
     # them local encoding on input, we would lose info writing them to
     # the cache.
-    for (name, (node, hist)) in sorted(cachetags.iteritems()):
+    for (name, (node, hist)) in sorted(pycompat.iteritems(cachetags)):
         for n in hist:
-            cachefile.write("%s %s\n" % (hex(n), name))
-        cachefile.write("%s %s\n" % (hex(node), name))
+            cachefile.write(b"%s %s\n" % (hex(n), name))
+        cachefile.write(b"%s %s\n" % (hex(node), name))
 
     try:
         cachefile.close()
     except (OSError, IOError):
         pass
 
-_fnodescachefile = 'cache/hgtagsfnodes1'
-_fnodesrecsize = 4 + 20 # changeset fragment + filenode
-_fnodesmissingrec = '\xff' * 24
+
+def tag(repo, names, node, message, local, user, date, editor=False):
+    '''tag a revision with one or more symbolic names.
+
+    names is a list of strings or, when adding a single tag, names may be a
+    string.
+
+    if local is True, the tags are stored in a per-repository file.
+    otherwise, they are stored in the .hgtags file, and a new
+    changeset is committed with the change.
+
+    keyword arguments:
+
+    local: whether to store tags in non-version-controlled file
+    (default False)
+
+    message: commit message to use if committing
+
+    user: name of user to use if committing
+
+    date: date tuple to use if committing'''
+
+    if not local:
+        m = matchmod.exact([b'.hgtags'])
+        st = repo.status(match=m, unknown=True, ignored=True)
+        if any(
+            (
+                st.modified,
+                st.added,
+                st.removed,
+                st.deleted,
+                st.unknown,
+                st.ignored,
+            )
+        ):
+            raise error.Abort(
+                _(b'working copy of .hgtags is changed'),
+                hint=_(b'please commit .hgtags manually'),
+            )
+
+    with repo.wlock():
+        repo.tags()  # instantiate the cache
+        _tag(repo, names, node, message, local, user, date, editor=editor)
+
+
+def _tag(
+    repo, names, node, message, local, user, date, extra=None, editor=False
+):
+    if isinstance(names, bytes):
+        names = (names,)
+
+    branches = repo.branchmap()
+    for name in names:
+        repo.hook(b'pretag', throw=True, node=hex(node), tag=name, local=local)
+        if name in branches:
+            repo.ui.warn(
+                _(b"warning: tag %s conflicts with existing branch name\n")
+                % name
+            )
+
+    def writetags(fp, names, munge, prevtags):
+        fp.seek(0, io.SEEK_END)
+        if prevtags and not prevtags.endswith(b'\n'):
+            fp.write(b'\n')
+        for name in names:
+            if munge:
+                m = munge(name)
+            else:
+                m = name
+
+            if repo._tagscache.tagtypes and name in repo._tagscache.tagtypes:
+                old = repo.tags().get(name, nullid)
+                fp.write(b'%s %s\n' % (hex(old), m))
+            fp.write(b'%s %s\n' % (hex(node), m))
+        fp.close()
+
+    prevtags = b''
+    if local:
+        try:
+            fp = repo.vfs(b'localtags', b'r+')
+        except IOError:
+            fp = repo.vfs(b'localtags', b'a')
+        else:
+            prevtags = fp.read()
+
+        # local tags are stored in the current charset
+        writetags(fp, names, None, prevtags)
+        for name in names:
+            repo.hook(b'tag', node=hex(node), tag=name, local=local)
+        return
+
+    try:
+        fp = repo.wvfs(b'.hgtags', b'rb+')
+    except IOError as e:
+        if e.errno != errno.ENOENT:
+            raise
+        fp = repo.wvfs(b'.hgtags', b'ab')
+    else:
+        prevtags = fp.read()
+
+    # committed tags are stored in UTF-8
+    writetags(fp, names, encoding.fromlocal, prevtags)
+
+    fp.close()
+
+    repo.invalidatecaches()
+
+    if b'.hgtags' not in repo.dirstate:
+        repo[None].add([b'.hgtags'])
+
+    m = matchmod.exact([b'.hgtags'])
+    tagnode = repo.commit(
+        message, user, date, extra=extra, match=m, editor=editor
+    )
+
+    for name in names:
+        repo.hook(b'tag', node=hex(node), tag=name, local=local)
+
+    return tagnode
+
+
+_fnodescachefile = b'hgtagsfnodes1'
+_fnodesrecsize = 4 + 20  # changeset fragment + filenode
+_fnodesmissingrec = b'\xff' * 24
+
 
 class hgtagsfnodescache(object):
     """Persistent cache mapping revisions to .hgtags filenodes.
@@ -422,6 +692,7 @@ class hgtagsfnodescache(object):
     Instances behave like lists. ``c[i]`` works where i is a rev or
     changeset node. Missing indexes are populated automatically on access.
     """
+
     def __init__(self, repo):
         assert repo.filtername is None
 
@@ -431,10 +702,11 @@ class hgtagsfnodescache(object):
         self.lookupcount = 0
         self.hitcount = 0
 
-        self._raw = array('c')
-
-        data = repo.vfs.tryread(_fnodescachefile)
-        self._raw.fromstring(data)
+        try:
+            data = repo.cachevfs.read(_fnodescachefile)
+        except (OSError, IOError):
+            data = b""
+        self._raw = bytearray(data)
 
         # The end state of self._raw is an array that is of the exact length
         # required to hold a record for every revision in the repository.
@@ -450,7 +722,7 @@ class hgtagsfnodescache(object):
 
         if rawlen < wantedlen:
             self._dirtyoffset = rawlen
-            self._raw.extend('\xff' * (wantedlen - rawlen))
+            self._raw.extend(b'\xff' * (wantedlen - rawlen))
         elif rawlen > wantedlen:
             # There's no easy way to truncate array instances. This seems
             # slightly less evil than copying a potentially large array slice.
@@ -469,13 +741,16 @@ class hgtagsfnodescache(object):
         If an .hgtags does not exist at the specified revision, nullid is
         returned.
         """
+        if node == nullid:
+            return nullid
+
         ctx = self._repo[node]
         rev = ctx.rev()
 
         self.lookupcount += 1
 
         offset = rev * _fnodesrecsize
-        record = self._raw[offset:offset + _fnodesrecsize].tostring()
+        record = b'%s' % self._raw[offset : offset + _fnodesrecsize]
         properprefix = node[0:4]
 
         # Validate and return existing entry.
@@ -493,12 +768,33 @@ class hgtagsfnodescache(object):
         if not computemissing:
             return None
 
-        # Populate missing entry.
-        try:
-            fnode = ctx.filenode('.hgtags')
-        except error.LookupError:
-            # No .hgtags file on this revision.
-            fnode = nullid
+        fnode = None
+        cl = self._repo.changelog
+        p1rev, p2rev = cl._uncheckedparentrevs(rev)
+        p1node = cl.node(p1rev)
+        p1fnode = self.getfnode(p1node, computemissing=False)
+        if p2rev != nullrev:
+            # There is some no-merge changeset where p1 is null and p2 is set
+            # Processing them as merge is just slower, but still gives a good
+            # result.
+            p2node = cl.node(p1rev)
+            p2fnode = self.getfnode(p2node, computemissing=False)
+            if p1fnode != p2fnode:
+                # we cannot rely on readfast because we don't know against what
+                # parent the readfast delta is computed
+                p1fnode = None
+        if p1fnode is not None:
+            mctx = ctx.manifestctx()
+            fnode = mctx.readfast().get(b'.hgtags')
+            if fnode is None:
+                fnode = p1fnode
+        if fnode is None:
+            # Populate missing entry.
+            try:
+                fnode = ctx.filenode(b'.hgtags')
+            except error.LookupError:
+                # No .hgtags file on this revision.
+                fnode = nullid
 
         self._writeentry(offset, properprefix, fnode)
         return fnode
@@ -516,10 +812,10 @@ class hgtagsfnodescache(object):
 
     def _writeentry(self, offset, prefix, fnode):
         # Slices on array instances only accept other array.
-        entry = array('c', prefix + fnode)
-        self._raw[offset:offset + _fnodesrecsize] = entry
+        entry = bytearray(prefix + fnode)
+        self._raw[offset : offset + _fnodesrecsize] = entry
         # self._dirtyoffset could be None.
-        self._dirtyoffset = min(self._dirtyoffset, offset) or 0
+        self._dirtyoffset = min(self._dirtyoffset or 0, offset or 0)
 
     def write(self):
         """Perform all necessary writes to cache file.
@@ -530,7 +826,7 @@ class hgtagsfnodescache(object):
         if self._dirtyoffset is None:
             return
 
-        data = self._raw[self._dirtyoffset:]
+        data = self._raw[self._dirtyoffset :]
         if not data:
             return
 
@@ -539,31 +835,37 @@ class hgtagsfnodescache(object):
         try:
             lock = repo.wlock(wait=False)
         except error.LockError:
-            repo.ui.log('tagscache',
-                        'not writing .hg/%s because lock cannot be acquired\n' %
-                        (_fnodescachefile))
+            repo.ui.log(
+                b'tagscache',
+                b'not writing .hg/cache/%s because '
+                b'lock cannot be acquired\n' % _fnodescachefile,
+            )
             return
 
         try:
-            f = repo.vfs.open(_fnodescachefile, 'ab')
+            f = repo.cachevfs.open(_fnodescachefile, b'ab')
             try:
                 # if the file has been truncated
                 actualoffset = f.tell()
                 if actualoffset < self._dirtyoffset:
                     self._dirtyoffset = actualoffset
-                    data = self._raw[self._dirtyoffset:]
+                    data = self._raw[self._dirtyoffset :]
                 f.seek(self._dirtyoffset)
                 f.truncate()
-                repo.ui.log('tagscache',
-                            'writing %d bytes to %s\n' % (
-                            len(data), _fnodescachefile))
+                repo.ui.log(
+                    b'tagscache',
+                    b'writing %d bytes to cache/%s\n'
+                    % (len(data), _fnodescachefile),
+                )
                 f.write(data)
                 self._dirtyoffset = None
             finally:
                 f.close()
         except (IOError, OSError) as inst:
-            repo.ui.log('tagscache',
-                        "couldn't write %s: %s\n" % (
-                        _fnodescachefile, inst))
+            repo.ui.log(
+                b'tagscache',
+                b"couldn't write cache/%s: %s\n"
+                % (_fnodescachefile, stringutil.forcebytestr(inst)),
+            )
         finally:
             lock.release()
