@@ -14,19 +14,21 @@ from ..node import (
     nullid,
     nullrev,
 )
+from ..pycompat import getattr
 from .. import (
     error,
     mdiff,
-    repository,
-)
-from ..utils import (
-    storageutil,
 )
+from ..interfaces import repository
+from ..utils import storageutil
+
 
 class basetestcase(unittest.TestCase):
-    if not getattr(unittest.TestCase, r'assertRaisesRegex', False):
-        assertRaisesRegex = (# camelcase-required
-            unittest.TestCase.assertRaisesRegexp)
+    if not getattr(unittest.TestCase, 'assertRaisesRegex', False):
+        assertRaisesRegex = (  # camelcase-required
+            unittest.TestCase.assertRaisesRegexp
+        )
+
 
 class ifileindextests(basetestcase):
     """Generic tests for the ifileindex interface.
@@ -36,10 +38,11 @@ class ifileindextests(basetestcase):
 
     Use ``makeifileindextests()`` to create an instance of this type.
     """
+
     def testempty(self):
         f = self._makefilefn()
-        self.assertEqual(len(f), 0, 'new file store has 0 length by default')
-        self.assertEqual(list(f), [], 'iter yields nothing by default')
+        self.assertEqual(len(f), 0, b'new file store has 0 length by default')
+        self.assertEqual(list(f), [], b'iter yields nothing by default')
 
         gen = iter(f)
         with self.assertRaises(StopIteration):
@@ -396,6 +399,7 @@ class ifileindextests(basetestcase):
         self.assertEqual(f.children(node4), [])
         self.assertEqual(f.children(node5), [])
 
+
 class ifiledatatests(basetestcase):
     """Generic tests for the ifiledata interface.
 
@@ -404,12 +408,15 @@ class ifiledatatests(basetestcase):
 
     Use ``makeifiledatatests()`` to create an instance of this type.
     """
+
     def testempty(self):
         f = self._makefilefn()
 
         self.assertEqual(f.storageinfo(), {})
-        self.assertEqual(f.storageinfo(revisionscount=True, trackedsize=True),
-                         {'revisionscount': 0, 'trackedsize': 0})
+        self.assertEqual(
+            f.storageinfo(revisionscount=True, trackedsize=True),
+            {b'revisionscount': 0, b'trackedsize': 0},
+        )
 
         self.assertEqual(f.size(nullrev), 0)
 
@@ -421,7 +428,7 @@ class ifiledatatests(basetestcase):
                 f.size(i)
 
         self.assertEqual(f.revision(nullid), b'')
-        self.assertEqual(f.revision(nullid, raw=True), b'')
+        self.assertEqual(f.rawdata(nullid), b'')
 
         with self.assertRaises(error.LookupError):
             f.revision(b'\x01' * 20)
@@ -464,8 +471,10 @@ class ifiledatatests(basetestcase):
             node = f.add(fulltext, None, tr, 0, nullid, nullid)
 
         self.assertEqual(f.storageinfo(), {})
-        self.assertEqual(f.storageinfo(revisionscount=True, trackedsize=True),
-                         {'revisionscount': 1, 'trackedsize': len(fulltext)})
+        self.assertEqual(
+            f.storageinfo(revisionscount=True, trackedsize=True),
+            {b'revisionscount': 1, b'trackedsize': len(fulltext)},
+        )
 
         self.assertEqual(f.size(0), len(fulltext))
 
@@ -473,7 +482,7 @@ class ifiledatatests(basetestcase):
             f.size(1)
 
         self.assertEqual(f.revision(node), fulltext)
-        self.assertEqual(f.revision(node, raw=True), fulltext)
+        self.assertEqual(f.rawdata(node), fulltext)
 
         self.assertEqual(f.read(node), fulltext)
 
@@ -533,9 +542,12 @@ class ifiledatatests(basetestcase):
         self.assertEqual(
             f.storageinfo(revisionscount=True, trackedsize=True),
             {
-                'revisionscount': 3,
-                'trackedsize': len(fulltext0) + len(fulltext1) + len(fulltext2),
-            })
+                b'revisionscount': 3,
+                b'trackedsize': len(fulltext0)
+                + len(fulltext1)
+                + len(fulltext2),
+            },
+        )
 
         self.assertEqual(f.size(0), len(fulltext0))
         self.assertEqual(f.size(1), len(fulltext1))
@@ -545,11 +557,11 @@ class ifiledatatests(basetestcase):
             f.size(3)
 
         self.assertEqual(f.revision(node0), fulltext0)
-        self.assertEqual(f.revision(node0, raw=True), fulltext0)
+        self.assertEqual(f.rawdata(node0), fulltext0)
         self.assertEqual(f.revision(node1), fulltext1)
-        self.assertEqual(f.revision(node1, raw=True), fulltext1)
+        self.assertEqual(f.rawdata(node1), fulltext1)
         self.assertEqual(f.revision(node2), fulltext2)
-        self.assertEqual(f.revision(node2, raw=True), fulltext2)
+        self.assertEqual(f.rawdata(node2), fulltext2)
 
         with self.assertRaises(error.LookupError):
             f.revision(b'\x01' * 20)
@@ -601,9 +613,10 @@ class ifiledatatests(basetestcase):
         self.assertEqual(rev.basenode, node0)
         self.assertIsNone(rev.baserevisionsize)
         self.assertIsNone(rev.revision)
-        self.assertEqual(rev.delta,
-                         b'\x00\x00\x00\x00\x00\x00\x04\x00\x00\x00\x04\x01' +
-                         fulltext1)
+        self.assertEqual(
+            rev.delta,
+            b'\x00\x00\x00\x00\x00\x00\x04\x00\x00\x00\x04\x01' + fulltext1,
+        )
 
         rev = next(gen)
 
@@ -614,9 +627,10 @@ class ifiledatatests(basetestcase):
         self.assertEqual(rev.basenode, node1)
         self.assertIsNone(rev.baserevisionsize)
         self.assertIsNone(rev.revision)
-        self.assertEqual(rev.delta,
-                         b'\x00\x00\x00\x00\x00\x00\x04\x01\x00\x00\x04\x02' +
-                         fulltext2)
+        self.assertEqual(
+            rev.delta,
+            b'\x00\x00\x00\x00\x00\x00\x04\x01\x00\x00\x04\x02' + fulltext2,
+        )
 
         with self.assertRaises(StopIteration):
             next(gen)
@@ -644,9 +658,10 @@ class ifiledatatests(basetestcase):
         self.assertEqual(rev.basenode, node0)
         self.assertIsNone(rev.baserevisionsize)
         self.assertIsNone(rev.revision)
-        self.assertEqual(rev.delta,
-                         b'\x00\x00\x00\x00\x00\x00\x04\x00\x00\x00\x04\x01' +
-                         fulltext1)
+        self.assertEqual(
+            rev.delta,
+            b'\x00\x00\x00\x00\x00\x00\x04\x00\x00\x00\x04\x01' + fulltext1,
+        )
 
         rev = next(gen)
 
@@ -657,27 +672,30 @@ class ifiledatatests(basetestcase):
         self.assertEqual(rev.basenode, node1)
         self.assertIsNone(rev.baserevisionsize)
         self.assertIsNone(rev.revision)
-        self.assertEqual(rev.delta,
-                         b'\x00\x00\x00\x00\x00\x00\x04\x01\x00\x00\x04\x02' +
-                         fulltext2)
+        self.assertEqual(
+            rev.delta,
+            b'\x00\x00\x00\x00\x00\x00\x04\x01\x00\x00\x04\x02' + fulltext2,
+        )
 
         with self.assertRaises(StopIteration):
             next(gen)
 
         # Unrecognized nodesorder value raises ProgrammingError.
         with self.assertRaises(error.ProgrammingError):
-            list(f.emitrevisions([], nodesorder='bad'))
+            list(f.emitrevisions([], nodesorder=b'bad'))
 
         # nodesorder=storage is recognized. But we can't test it thoroughly
         # because behavior is storage-dependent.
-        res = list(f.emitrevisions([node2, node1, node0],
-                                         nodesorder='storage'))
+        res = list(
+            f.emitrevisions([node2, node1, node0], nodesorder=b'storage')
+        )
         self.assertEqual(len(res), 3)
         self.assertEqual({o.node for o in res}, {node0, node1, node2})
 
         # nodesorder=nodes forces the order.
-        gen = f.emitrevisions([node2, node0], nodesorder='nodes',
-                              revisiondata=True)
+        gen = f.emitrevisions(
+            [node2, node0], nodesorder=b'nodes', revisiondata=True
+        )
 
         rev = next(gen)
         self.assertEqual(rev.node, node2)
@@ -717,16 +735,18 @@ class ifiledatatests(basetestcase):
         self.assertEqual(rev.basenode, node1)
         self.assertIsNone(rev.baserevisionsize)
         self.assertIsNone(rev.revision)
-        self.assertEqual(rev.delta,
-                         b'\x00\x00\x00\x00\x00\x00\x04\x01\x00\x00\x04\x02' +
-                         fulltext2)
+        self.assertEqual(
+            rev.delta,
+            b'\x00\x00\x00\x00\x00\x00\x04\x01\x00\x00\x04\x02' + fulltext2,
+        )
 
         with self.assertRaises(StopIteration):
             next(gen)
 
         # assumehaveparentrevisions=True allows delta against initial revision.
-        gen = f.emitrevisions([node2, node1],
-                              revisiondata=True, assumehaveparentrevisions=True)
+        gen = f.emitrevisions(
+            [node2, node1], revisiondata=True, assumehaveparentrevisions=True
+        )
 
         rev = next(gen)
         self.assertEqual(rev.node, node1)
@@ -735,13 +755,16 @@ class ifiledatatests(basetestcase):
         self.assertEqual(rev.basenode, node0)
         self.assertIsNone(rev.baserevisionsize)
         self.assertIsNone(rev.revision)
-        self.assertEqual(rev.delta,
-                         b'\x00\x00\x00\x00\x00\x00\x04\x00\x00\x00\x04\x01' +
-                         fulltext1)
+        self.assertEqual(
+            rev.delta,
+            b'\x00\x00\x00\x00\x00\x00\x04\x00\x00\x00\x04\x01' + fulltext1,
+        )
 
         # forceprevious=True forces a delta against the previous revision.
         # Special case for initial revision.
-        gen = f.emitrevisions([node0], revisiondata=True, deltaprevious=True)
+        gen = f.emitrevisions(
+            [node0], revisiondata=True, deltamode=repository.CG_DELTAMODE_PREV
+        )
 
         rev = next(gen)
         self.assertEqual(rev.node, node0)
@@ -750,15 +773,19 @@ class ifiledatatests(basetestcase):
         self.assertEqual(rev.basenode, nullid)
         self.assertIsNone(rev.baserevisionsize)
         self.assertIsNone(rev.revision)
-        self.assertEqual(rev.delta,
-                         b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x04\x00' +
-                         fulltext0)
+        self.assertEqual(
+            rev.delta,
+            b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x04\x00' + fulltext0,
+        )
 
         with self.assertRaises(StopIteration):
             next(gen)
 
-        gen = f.emitrevisions([node0, node2], revisiondata=True,
-                              deltaprevious=True)
+        gen = f.emitrevisions(
+            [node0, node2],
+            revisiondata=True,
+            deltamode=repository.CG_DELTAMODE_PREV,
+        )
 
         rev = next(gen)
         self.assertEqual(rev.node, node0)
@@ -767,9 +794,10 @@ class ifiledatatests(basetestcase):
         self.assertEqual(rev.basenode, nullid)
         self.assertIsNone(rev.baserevisionsize)
         self.assertIsNone(rev.revision)
-        self.assertEqual(rev.delta,
-                         b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x04\x00' +
-                         fulltext0)
+        self.assertEqual(
+            rev.delta,
+            b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x04\x00' + fulltext0,
+        )
 
         rev = next(gen)
         self.assertEqual(rev.node, node2)
@@ -795,17 +823,21 @@ class ifiledatatests(basetestcase):
             b'copyrev': b'b' * 40,
         }
 
-        stored1 = b''.join([
-            b'\x01\ncopy: source0\n',
-            b'copyrev: aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\x01\n',
-            fulltext1,
-        ])
+        stored1 = b''.join(
+            [
+                b'\x01\ncopy: source0\n',
+                b'copyrev: aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n\x01\n',
+                fulltext1,
+            ]
+        )
 
-        stored2 = b''.join([
-            b'\x01\ncopy: source1\n',
-            b'copyrev: bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb\n\x01\n',
-            fulltext2,
-        ])
+        stored2 = b''.join(
+            [
+                b'\x01\ncopy: source1\n',
+                b'copyrev: bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb\n\x01\n',
+                fulltext2,
+            ]
+        )
 
         f = self._makefilefn()
         with self._maketransactionfn() as tr:
@@ -818,9 +850,9 @@ class ifiledatatests(basetestcase):
         self.assertEqual(f.size(2), len(fulltext2))
 
         self.assertEqual(f.revision(node1), stored1)
-        self.assertEqual(f.revision(node1, raw=True), stored1)
+        self.assertEqual(f.rawdata(node1), stored1)
         self.assertEqual(f.revision(node2), stored2)
-        self.assertEqual(f.revision(node2, raw=True), stored2)
+        self.assertEqual(f.rawdata(node2), stored2)
 
         self.assertEqual(f.read(node1), fulltext1)
         self.assertEqual(f.read(node2), fulltext2)
@@ -844,11 +876,13 @@ class ifiledatatests(basetestcase):
             b'copy': b'source0',
             b'copyrev': b'b' * 40,
         }
-        stored1 = b''.join([
-            b'\x01\ncopy: source0\n',
-            b'copyrev: %s\n' % (b'b' * 40),
-            b'\x01\n\x01\nbar',
-        ])
+        stored1 = b''.join(
+            [
+                b'\x01\ncopy: source0\n',
+                b'copyrev: %s\n' % (b'b' * 40),
+                b'\x01\n\x01\nbar',
+            ]
+        )
 
         f = self._makefilefn()
         with self._maketransactionfn() as tr:
@@ -861,10 +895,10 @@ class ifiledatatests(basetestcase):
         self.assertEqual(f.size(1), len(fulltext1))
 
         self.assertEqual(f.revision(node0), stored0)
-        self.assertEqual(f.revision(node0, raw=True), stored0)
+        self.assertEqual(f.rawdata(node0), stored0)
 
         self.assertEqual(f.revision(node1), stored1)
-        self.assertEqual(f.revision(node1, raw=True), stored1)
+        self.assertEqual(f.rawdata(node1), stored1)
 
         self.assertEqual(f.read(node0), fulltext0)
         self.assertEqual(f.read(node1), fulltext1)
@@ -885,8 +919,9 @@ class ifiledatatests(basetestcase):
             node0 = f.add(fulltext0, None, tr, 0, nullid, nullid)
             node1 = b'\xaa' * 20
 
-            self._addrawrevisionfn(f, tr, node1, node0, nullid, 1,
-                                   rawtext=fulltext1)
+            self._addrawrevisionfn(
+                f, tr, node1, node0, nullid, 1, rawtext=fulltext1
+            )
 
         self.assertEqual(len(f), 2)
         self.assertEqual(f.parents(node1), (node0, nullid))
@@ -895,10 +930,10 @@ class ifiledatatests(basetestcase):
         with self.assertRaises(error.StorageError):
             f.revision(node1)
 
-        # raw=True still verifies because there are no special storage
+        # rawdata() still verifies because there are no special storage
         # settings.
         with self.assertRaises(error.StorageError):
-            f.revision(node1, raw=True)
+            f.rawdata(node1)
 
         # read() behaves like revision().
         with self.assertRaises(error.StorageError):
@@ -908,7 +943,7 @@ class ifiledatatests(basetestcase):
         # reading/validating the fulltext to return rename metadata.
 
     def testbadnoderevisionraw(self):
-        # Like above except we test revision(raw=True) first to isolate
+        # Like above except we test rawdata() first to isolate
         # revision caching behavior.
         f = self._makefilefn()
 
@@ -919,16 +954,17 @@ class ifiledatatests(basetestcase):
             node0 = f.add(fulltext0, None, tr, 0, nullid, nullid)
             node1 = b'\xaa' * 20
 
-            self._addrawrevisionfn(f, tr, node1, node0, nullid, 1,
-                                   rawtext=fulltext1)
+            self._addrawrevisionfn(
+                f, tr, node1, node0, nullid, 1, rawtext=fulltext1
+            )
 
         with self.assertRaises(error.StorageError):
-            f.revision(node1, raw=True)
+            f.rawdata(node1)
 
         with self.assertRaises(error.StorageError):
-            f.revision(node1, raw=True)
+            f.rawdata(node1)
 
-    def testbadnoderevisionraw(self):
+    def testbadnoderevision(self):
         # Like above except we test read() first to isolate revision caching
         # behavior.
         f = self._makefilefn()
@@ -940,8 +976,9 @@ class ifiledatatests(basetestcase):
             node0 = f.add(fulltext0, None, tr, 0, nullid, nullid)
             node1 = b'\xaa' * 20
 
-            self._addrawrevisionfn(f, tr, node1, node0, nullid, 1,
-                                   rawtext=fulltext1)
+            self._addrawrevisionfn(
+                f, tr, node1, node0, nullid, 1, rawtext=fulltext1
+            )
 
         with self.assertRaises(error.StorageError):
             f.read(node1)
@@ -960,8 +997,9 @@ class ifiledatatests(basetestcase):
             node0 = f.add(fulltext0, None, tr, 0, nullid, nullid)
             node1 = b'\xaa' * 20
 
-            self._addrawrevisionfn(f, tr, node1, node0, nullid, 1,
-                                   rawtext=fulltext1)
+            self._addrawrevisionfn(
+                f, tr, node1, node0, nullid, 1, rawtext=fulltext1
+            )
 
         with self.assertRaises(error.StorageError):
             f.read(node1)
@@ -970,8 +1008,9 @@ class ifiledatatests(basetestcase):
 
         with self._maketransactionfn() as tr:
             delta = mdiff.textdiff(fulltext1, fulltext2)
-            self._addrawrevisionfn(f, tr, node2, node1, nullid,
-                                   2, delta=(1, delta))
+            self._addrawrevisionfn(
+                f, tr, node2, node1, nullid, 2, delta=(1, delta)
+            )
 
         self.assertEqual(len(f), 3)
 
@@ -982,9 +1021,7 @@ class ifiledatatests(basetestcase):
     def testcensored(self):
         f = self._makefilefn()
 
-        stored1 = storageutil.packmeta({
-            b'censored': b'tombstone',
-        }, b'')
+        stored1 = storageutil.packmeta({b'censored': b'tombstone',}, b'')
 
         with self._maketransactionfn() as tr:
             node0 = f.add(b'foo', None, tr, 0, nullid, nullid)
@@ -992,8 +1029,9 @@ class ifiledatatests(basetestcase):
             # The node value doesn't matter since we can't verify it.
             node1 = b'\xbb' * 20
 
-            self._addrawrevisionfn(f, tr, node1, node0, nullid, 1, stored1,
-                                   censored=True)
+            self._addrawrevisionfn(
+                f, tr, node1, node0, nullid, 1, stored1, censored=True
+            )
 
         self.assertTrue(f.iscensored(1))
 
@@ -1001,20 +1039,18 @@ class ifiledatatests(basetestcase):
             f.revision(1)
 
         with self.assertRaises(error.CensoredNodeError):
-            f.revision(1, raw=True)
+            f.rawdata(1)
 
         with self.assertRaises(error.CensoredNodeError):
             f.read(1)
 
     def testcensoredrawrevision(self):
-        # Like above, except we do the revision(raw=True) request first to
+        # Like above, except we do the rawdata() request first to
         # isolate revision caching behavior.
 
         f = self._makefilefn()
 
-        stored1 = storageutil.packmeta({
-            b'censored': b'tombstone',
-        }, b'')
+        stored1 = storageutil.packmeta({b'censored': b'tombstone',}, b'')
 
         with self._maketransactionfn() as tr:
             node0 = f.add(b'foo', None, tr, 0, nullid, nullid)
@@ -1022,11 +1058,13 @@ class ifiledatatests(basetestcase):
             # The node value doesn't matter since we can't verify it.
             node1 = b'\xbb' * 20
 
-            self._addrawrevisionfn(f, tr, node1, node0, nullid, 1, stored1,
-                                   censored=True)
+            self._addrawrevisionfn(
+                f, tr, node1, node0, nullid, 1, stored1, censored=True
+            )
 
         with self.assertRaises(error.CensoredNodeError):
-            f.revision(1, raw=True)
+            f.rawdata(1)
+
 
 class ifilemutationtests(basetestcase):
     """Generic tests for the ifilemutation interface.
@@ -1036,6 +1074,7 @@ class ifilemutationtests(basetestcase):
 
     Use ``makeifilemutationtests()`` to create an instance of this type.
     """
+
     def testaddnoop(self):
         f = self._makefilefn()
         with self._maketransactionfn() as tr:
@@ -1070,6 +1109,7 @@ class ifilemutationtests(basetestcase):
         f = self._makefilefn()
 
         callbackargs = []
+
         def cb(*args, **kwargs):
             callbackargs.append((args, kwargs))
 
@@ -1098,9 +1138,13 @@ class ifilemutationtests(basetestcase):
         with self._maketransactionfn() as tr:
             nodes = f.addgroup(deltas, linkmapper, tr, addrevisioncb=cb)
 
-        self.assertEqual(nodes, [
-            b'\x49\xd8\xcb\xb1\x5c\xe2\x57\x92\x04\x47'
-            b'\x00\x6b\x46\x97\x8b\x7a\xf9\x80\xa9\x79'])
+        self.assertEqual(
+            nodes,
+            [
+                b'\x49\xd8\xcb\xb1\x5c\xe2\x57\x92\x04\x47'
+                b'\x00\x6b\x46\x97\x8b\x7a\xf9\x80\xa9\x79'
+            ],
+        )
 
         self.assertEqual(len(callbackargs), 1)
         self.assertEqual(callbackargs[0][0][1], nodes[0])
@@ -1146,9 +1190,7 @@ class ifilemutationtests(basetestcase):
         # Attempt to apply a delta made against a censored revision.
         f = self._makefilefn()
 
-        stored1 = storageutil.packmeta({
-            b'censored': b'tombstone',
-        }, b'')
+        stored1 = storageutil.packmeta({b'censored': b'tombstone',}, b'')
 
         with self._maketransactionfn() as tr:
             node0 = f.add(b'foo\n' * 30, None, tr, 0, nullid, nullid)
@@ -1156,8 +1198,9 @@ class ifilemutationtests(basetestcase):
             # The node value doesn't matter since we can't verify it.
             node1 = b'\xbb' * 20
 
-            self._addrawrevisionfn(f, tr, node1, node0, nullid, 1, stored1,
-                                   censored=True)
+            self._addrawrevisionfn(
+                f, tr, node1, node0, nullid, 1, stored1, censored=True
+            )
 
         delta = mdiff.textdiff(b'bar\n' * 30, (b'bar\n' * 30) + b'baz\n')
         deltas = [(b'\xcc' * 20, node1, nullid, b'\x01' * 20, node1, delta, 0)]
@@ -1303,6 +1346,7 @@ class ifilemutationtests(basetestcase):
         with self.assertRaises(error.LookupError):
             f.rev(node1)
 
+
 def makeifileindextests(makefilefn, maketransactionfn, addrawrevisionfn):
     """Create a unittest.TestCase class suitable for testing file storage.
 
@@ -1322,24 +1366,26 @@ def makeifileindextests(makefilefn, maketransactionfn, addrawrevisionfn):
     should find and run it automatically.
     """
     d = {
-        r'_makefilefn': makefilefn,
-        r'_maketransactionfn': maketransactionfn,
-        r'_addrawrevisionfn': addrawrevisionfn,
+        '_makefilefn': makefilefn,
+        '_maketransactionfn': maketransactionfn,
+        '_addrawrevisionfn': addrawrevisionfn,
     }
-    return type(r'ifileindextests', (ifileindextests,), d)
+    return type('ifileindextests', (ifileindextests,), d)
+
 
 def makeifiledatatests(makefilefn, maketransactionfn, addrawrevisionfn):
     d = {
-        r'_makefilefn': makefilefn,
-        r'_maketransactionfn': maketransactionfn,
-        r'_addrawrevisionfn': addrawrevisionfn,
+        '_makefilefn': makefilefn,
+        '_maketransactionfn': maketransactionfn,
+        '_addrawrevisionfn': addrawrevisionfn,
     }
-    return type(r'ifiledatatests', (ifiledatatests,), d)
+    return type('ifiledatatests', (ifiledatatests,), d)
+
 
 def makeifilemutationtests(makefilefn, maketransactionfn, addrawrevisionfn):
     d = {
-        r'_makefilefn': makefilefn,
-        r'_maketransactionfn': maketransactionfn,
-        r'_addrawrevisionfn': addrawrevisionfn,
+        '_makefilefn': makefilefn,
+        '_maketransactionfn': maketransactionfn,
+        '_addrawrevisionfn': addrawrevisionfn,
     }
-    return type(r'ifilemutationtests', (ifilemutationtests,), d)
+    return type('ifilemutationtests', (ifilemutationtests,), d)
