@@ -10,19 +10,25 @@
 //! Ideally, we should use an Index entirely implemented in Rust,
 //! but this will take some time to get there.
 
-use cpython::{PyClone, PyObject, PyResult, Python};
+use cpython::{exc::ImportError, PyClone, PyErr, PyObject, PyResult, Python};
 use hg::{Graph, GraphError, Revision, WORKING_DIRECTORY_REVISION};
 use libc::c_int;
 
-py_capsule_fn!(
-    from mercurial.cext.parsers import index_get_parents_CAPI
-        as get_parents_capi
-        signature (
-            index: *mut RawPyObject,
-            rev: c_int,
-            ps: *mut [c_int; 2],
-        ) -> c_int
-);
+const REVLOG_CABI_VERSION: c_int = 1;
+
+#[repr(C)]
+pub struct Revlog_CAPI {
+    abi_version: c_int,
+    index_parents: unsafe extern "C" fn(
+        index: *mut revlog_capi::RawPyObject,
+        rev: c_int,
+        ps: *mut [c_int; 2],
+    ) -> c_int,
+}
+
+py_capsule!(
+    from mercurial.cext.parsers import revlog_CAPI
+        as revlog_capi for Revlog_CAPI);
 
 /// A `Graph` backed up by objects and functions from revlog.c
 ///
@@ -58,16 +64,32 @@ py_capsule_fn!(
 /// mechanisms in other contexts.
 pub struct Index {
     index: PyObject,
-    parents: get_parents_capi::CapsuleFn,
+    capi: &'static Revlog_CAPI,
 }
 
 impl Index {
     pub fn new(py: Python, index: PyObject) -> PyResult<Self> {
+        let capi = unsafe { revlog_capi::retrieve(py)? };
+        if capi.abi_version != REVLOG_CABI_VERSION {
+            return Err(PyErr::new::<ImportError, _>(
+                py,
+                format!(
+                    "ABI version mismatch: the C ABI revlog version {} \
+                     does not match the {} expected by Rust hg-cpython",
+                    capi.abi_version, REVLOG_CABI_VERSION
+                ),
+            ));
+        }
         Ok(Index {
             index: index,
-            parents: get_parents_capi::retrieve(py)?,
+            capi: capi,
         })
     }
+
+    /// return a reference to the CPython Index object in this Struct
+    pub fn inner(&self) -> &PyObject {
+        &self.index
+    }
 }
 
 impl Clone for Index {
@@ -75,7 +97,16 @@ impl Clone for Index {
         let guard = Python::acquire_gil();
         Index {
             index: self.index.clone_ref(guard.python()),
-            parents: self.parents.clone(),
+            capi: self.capi,
+        }
+    }
+}
+
+impl PyClone for Index {
+    fn clone_ref(&self, py: Python) -> Self {
+        Index {
+            index: self.index.clone_ref(py),
+            capi: self.capi,
         }
     }
 }
@@ -88,7 +119,7 @@ impl Graph for Index {
         }
         let mut res: [c_int; 2] = [0; 2];
         let code = unsafe {
-            (self.parents)(
+            (self.capi.index_parents)(
                 self.index.as_ptr(),
                 rev as c_int,
                 &mut res as *mut [c_int; 2],
