@@ -8,12 +8,13 @@ from mercurial import (
     filemerge,
 )
 
-def failfilemerge(filemergefn,
-                  premerge, repo, wctx, mynode, orig, fcd, fco, fca,
-                  labels=None):
+
+def failfilemerge(
+    filemergefn, premerge, repo, wctx, mynode, orig, fcd, fco, fca, labels=None
+):
     raise error.Abort("^C")
     return filemergefn(premerge, repo, mynode, orig, fcd, fco, fca, labels)
 
+
 def extsetup(ui):
-    extensions.wrapfunction(filemerge, '_filemerge',
-                            failfilemerge)
+    extensions.wrapfunction(filemerge, '_filemerge', failfilemerge)
