@@ -12,9 +12,15 @@ from mercurial import (
 cmdtable = {}
 command = registrar.command(cmdtable)
 
-@command(b'fakemergerecord',
-         [(b'X', b'mandatory', None, b'add a fake mandatory record'),
-          (b'x', b'advisory', None, b'add a fake advisory record')], '')
+
+@command(
+    b'fakemergerecord',
+    [
+        (b'X', b'mandatory', None, b'add a fake mandatory record'),
+        (b'x', b'advisory', None, b'add a fake advisory record'),
+    ],
+    '',
+)
 def fakemergerecord(ui, repo, *pats, **opts):
     with repo.wlock():
         ms = merge.mergestate.read(repo)
