@@ -8,8 +8,18 @@
 #   seq START STEP STOP   [START, STOP] stepping by STEP
 
 from __future__ import absolute_import, print_function
+import os
 import sys
 
+try:
+    import msvcrt
+
+    msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)
+    msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
+    msvcrt.setmode(sys.stderr.fileno(), os.O_BINARY)
+except ImportError:
+    pass
+
 if sys.version_info[0] >= 3:
     xrange = range
 
