@@ -6,17 +6,17 @@ from mercurial.hgweb import request as requestmod
 from mercurial import error
 
 DEFAULT_ENV = {
-    r'REQUEST_METHOD': r'GET',
-    r'SERVER_NAME': r'testserver',
-    r'SERVER_PORT': r'80',
-    r'SERVER_PROTOCOL': r'http',
-    r'wsgi.version': (1, 0),
-    r'wsgi.url_scheme': r'http',
-    r'wsgi.input': None,
-    r'wsgi.errors': None,
-    r'wsgi.multithread': False,
-    r'wsgi.multiprocess': True,
-    r'wsgi.run_once': False,
+    'REQUEST_METHOD': 'GET',
+    'SERVER_NAME': 'testserver',
+    'SERVER_PORT': '80',
+    'SERVER_PROTOCOL': 'http',
+    'wsgi.version': (1, 0),
+    'wsgi.url_scheme': 'http',
+    'wsgi.input': None,
+    'wsgi.errors': None,
+    'wsgi.multithread': False,
+    'wsgi.multiprocess': True,
+    'wsgi.run_once': False,
 }
 
 
@@ -49,7 +49,7 @@ class ParseRequestTests(unittest.TestCase):
         self.assertEqual(len(r.headers), 0)
 
     def testcustomport(self):
-        r = parse(DEFAULT_ENV, extra={r'SERVER_PORT': r'8000',})
+        r = parse(DEFAULT_ENV, extra={'SERVER_PORT': '8000',})
 
         self.assertEqual(r.url, b'http://testserver:8000')
         self.assertEqual(r.baseurl, b'http://testserver:8000')
@@ -58,7 +58,7 @@ class ParseRequestTests(unittest.TestCase):
 
         r = parse(
             DEFAULT_ENV,
-            extra={r'SERVER_PORT': r'4000', r'wsgi.url_scheme': r'https',},
+            extra={'SERVER_PORT': '4000', 'wsgi.url_scheme': 'https',},
         )
 
         self.assertEqual(r.url, b'https://testserver:4000')
@@ -67,7 +67,7 @@ class ParseRequestTests(unittest.TestCase):
         self.assertEqual(r.advertisedbaseurl, b'https://testserver:4000')
 
     def testhttphost(self):
-        r = parse(DEFAULT_ENV, extra={r'HTTP_HOST': r'altserver',})
+        r = parse(DEFAULT_ENV, extra={'HTTP_HOST': 'altserver',})
 
         self.assertEqual(r.url, b'http://altserver')
         self.assertEqual(r.baseurl, b'http://altserver')
@@ -75,7 +75,7 @@ class ParseRequestTests(unittest.TestCase):
         self.assertEqual(r.advertisedbaseurl, b'http://testserver')
 
     def testscriptname(self):
-        r = parse(DEFAULT_ENV, extra={r'SCRIPT_NAME': r'',})
+        r = parse(DEFAULT_ENV, extra={'SCRIPT_NAME': '',})
 
         self.assertEqual(r.url, b'http://testserver')
         self.assertEqual(r.baseurl, b'http://testserver')
@@ -85,7 +85,7 @@ class ParseRequestTests(unittest.TestCase):
         self.assertEqual(r.dispatchparts, [])
         self.assertIsNone(r.dispatchpath)
 
-        r = parse(DEFAULT_ENV, extra={r'SCRIPT_NAME': r'/script',})
+        r = parse(DEFAULT_ENV, extra={'SCRIPT_NAME': '/script',})
 
         self.assertEqual(r.url, b'http://testserver/script')
         self.assertEqual(r.baseurl, b'http://testserver')
@@ -95,7 +95,7 @@ class ParseRequestTests(unittest.TestCase):
         self.assertEqual(r.dispatchparts, [])
         self.assertIsNone(r.dispatchpath)
 
-        r = parse(DEFAULT_ENV, extra={r'SCRIPT_NAME': r'/multiple words',})
+        r = parse(DEFAULT_ENV, extra={'SCRIPT_NAME': '/multiple words',})
 
         self.assertEqual(r.url, b'http://testserver/multiple%20words')
         self.assertEqual(r.baseurl, b'http://testserver')
@@ -106,7 +106,7 @@ class ParseRequestTests(unittest.TestCase):
         self.assertIsNone(r.dispatchpath)
 
     def testpathinfo(self):
-        r = parse(DEFAULT_ENV, extra={r'PATH_INFO': r'',})
+        r = parse(DEFAULT_ENV, extra={'PATH_INFO': '',})
 
         self.assertEqual(r.url, b'http://testserver')
         self.assertEqual(r.baseurl, b'http://testserver')
@@ -116,7 +116,7 @@ class ParseRequestTests(unittest.TestCase):
         self.assertEqual(r.dispatchparts, [])
         self.assertEqual(r.dispatchpath, b'')
 
-        r = parse(DEFAULT_ENV, extra={r'PATH_INFO': r'/pathinfo',})
+        r = parse(DEFAULT_ENV, extra={'PATH_INFO': '/pathinfo',})
 
         self.assertEqual(r.url, b'http://testserver/pathinfo')
         self.assertEqual(r.baseurl, b'http://testserver')
@@ -126,7 +126,7 @@ class ParseRequestTests(unittest.TestCase):
         self.assertEqual(r.dispatchparts, [b'pathinfo'])
         self.assertEqual(r.dispatchpath, b'pathinfo')
 
-        r = parse(DEFAULT_ENV, extra={r'PATH_INFO': r'/one/two/',})
+        r = parse(DEFAULT_ENV, extra={'PATH_INFO': '/one/two/',})
 
         self.assertEqual(r.url, b'http://testserver/one/two/')
         self.assertEqual(r.baseurl, b'http://testserver')
@@ -139,7 +139,7 @@ class ParseRequestTests(unittest.TestCase):
     def testscriptandpathinfo(self):
         r = parse(
             DEFAULT_ENV,
-            extra={r'SCRIPT_NAME': r'/script', r'PATH_INFO': r'/pathinfo',},
+            extra={'SCRIPT_NAME': '/script', 'PATH_INFO': '/pathinfo',},
         )
 
         self.assertEqual(r.url, b'http://testserver/script/pathinfo')
@@ -153,8 +153,8 @@ class ParseRequestTests(unittest.TestCase):
         r = parse(
             DEFAULT_ENV,
             extra={
-                r'SCRIPT_NAME': r'/script1/script2',
-                r'PATH_INFO': r'/path1/path2',
+                'SCRIPT_NAME': '/script1/script2',
+                'PATH_INFO': '/path1/path2',
             },
         )
 
@@ -173,9 +173,9 @@ class ParseRequestTests(unittest.TestCase):
         r = parse(
             DEFAULT_ENV,
             extra={
-                r'HTTP_HOST': r'hostserver',
-                r'SCRIPT_NAME': r'/script',
-                r'PATH_INFO': r'/pathinfo',
+                'HTTP_HOST': 'hostserver',
+                'SCRIPT_NAME': '/script',
+                'PATH_INFO': '/pathinfo',
             },
         )
 
@@ -208,7 +208,7 @@ class ParseRequestTests(unittest.TestCase):
             parse(
                 DEFAULT_ENV,
                 reponame=b'repo',
-                extra={r'PATH_INFO': r'/pathinfo',},
+                extra={'PATH_INFO': '/pathinfo',},
             )
 
         with self.assertRaisesRegex(
@@ -217,13 +217,13 @@ class ParseRequestTests(unittest.TestCase):
             parse(
                 DEFAULT_ENV,
                 reponame=b'repo',
-                extra={r'PATH_INFO': r'/repoextra/path',},
+                extra={'PATH_INFO': '/repoextra/path',},
             )
 
         r = parse(
             DEFAULT_ENV,
             reponame=b'repo',
-            extra={r'PATH_INFO': r'/repo/path1/path2',},
+            extra={'PATH_INFO': '/repo/path1/path2',},
         )
 
         self.assertEqual(r.url, b'http://testserver/repo/path1/path2')
@@ -238,7 +238,7 @@ class ParseRequestTests(unittest.TestCase):
         r = parse(
             DEFAULT_ENV,
             reponame=b'prefix/repo',
-            extra={r'PATH_INFO': r'/prefix/repo/path1/path2',},
+            extra={'PATH_INFO': '/prefix/repo/path1/path2',},
         )
 
         self.assertEqual(r.url, b'http://testserver/prefix/repo/path1/path2')
@@ -307,7 +307,7 @@ class ParseRequestTests(unittest.TestCase):
         r = parse(
             DEFAULT_ENV,
             altbaseurl=b'http://altserver',
-            extra={r'PATH_INFO': r'/path1/path2',},
+            extra={'PATH_INFO': '/path1/path2',},
         )
         self.assertEqual(r.url, b'http://testserver/path1/path2')
         self.assertEqual(r.baseurl, b'http://testserver')
@@ -347,7 +347,7 @@ class ParseRequestTests(unittest.TestCase):
         r = parse(
             DEFAULT_ENV,
             altbaseurl=b'http://altserver/altpath',
-            extra={r'PATH_INFO': r'/path1/path2',},
+            extra={'PATH_INFO': '/path1/path2',},
         )
         self.assertEqual(r.url, b'http://testserver/path1/path2')
         self.assertEqual(r.baseurl, b'http://testserver')
@@ -365,7 +365,7 @@ class ParseRequestTests(unittest.TestCase):
         r = parse(
             DEFAULT_ENV,
             altbaseurl=b'http://altserver/altpath/',
-            extra={r'PATH_INFO': r'/path1/path2',},
+            extra={'PATH_INFO': '/path1/path2',},
         )
         self.assertEqual(r.url, b'http://testserver/path1/path2')
         self.assertEqual(r.baseurl, b'http://testserver')
@@ -383,7 +383,7 @@ class ParseRequestTests(unittest.TestCase):
         r = parse(
             DEFAULT_ENV,
             altbaseurl=b'http://altserver',
-            extra={r'SCRIPT_NAME': r'/script', r'PATH_INFO': r'/path1/path2',},
+            extra={'SCRIPT_NAME': '/script', 'PATH_INFO': '/path1/path2',},
         )
         self.assertEqual(r.url, b'http://testserver/script/path1/path2')
         self.assertEqual(r.baseurl, b'http://testserver')
@@ -399,7 +399,7 @@ class ParseRequestTests(unittest.TestCase):
         r = parse(
             DEFAULT_ENV,
             altbaseurl=b'http://altserver/altroot',
-            extra={r'SCRIPT_NAME': r'/script', r'PATH_INFO': r'/path1/path2',},
+            extra={'SCRIPT_NAME': '/script', 'PATH_INFO': '/path1/path2',},
         )
         self.assertEqual(r.url, b'http://testserver/script/path1/path2')
         self.assertEqual(r.baseurl, b'http://testserver')
@@ -418,10 +418,7 @@ class ParseRequestTests(unittest.TestCase):
             DEFAULT_ENV,
             reponame=b'repo',
             altbaseurl=b'http://altserver/altroot',
-            extra={
-                r'SCRIPT_NAME': r'/script',
-                r'PATH_INFO': r'/repo/path1/path2',
-            },
+            extra={'SCRIPT_NAME': '/script', 'PATH_INFO': '/repo/path1/path2',},
         )
 
         self.assertEqual(r.url, b'http://testserver/script/repo/path1/path2')
