@@ -2,16 +2,24 @@
 
 from __future__ import absolute_import
 
-def reposetup(ui, repo):
 
+def reposetup(ui, repo):
     def reportphasemove(tr):
         for rev, move in sorted(tr.changes[b'phases'].items()):
             if move[0] is None:
-                ui.write((b'test-debug-phase: new rev %d:  x -> %d\n'
-                          % (rev, move[1])))
+                ui.write(
+                    (
+                        b'test-debug-phase: new rev %d:  x -> %d\n'
+                        % (rev, move[1])
+                    )
+                )
             else:
-                ui.write((b'test-debug-phase: move rev %d: %d -> %d\n'
-                          % (rev, move[0], move[1])))
+                ui.write(
+                    (
+                        b'test-debug-phase: move rev %d: %d -> %d\n'
+                        % (rev, move[0], move[1])
+                    )
+                )
 
     class reportphaserepo(repo.__class__):
         def transaction(self, *args, **kwargs):
